# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import get_resource_service
from superdesk.utc import utcnow, utc_to_local
from collections import namedtuple
from subprocess import check_call, PIPE
from flask import current_app as app
import pytz
from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta
from math import floor
import re
import logging

logger = logging.getLogger(__name__)


mime_types = [
    'image/png',
    'image/jpeg',
    'image/gif',
    'application/pdf',
    'image/svg+xml',
    'text/csv',
    'text/html'
]

MIME_TYPES = namedtuple('MIME_TYPES', [
    'PNG',
    'JPEG',
    'GIF',
    'PDF',
    'SVG',
    'CSV',
    'HTML'
])(*mime_types)

date_filters = [
    # ABSOLUTE
    'range',
    'day',

    # HOURS
    'relative_hours',

    # DAYS
    'relative_days',
    'yesterday',
    'today',

    # WEEKS
    'relative_weeks',
    'last_week',
    'this_week',

    # MONTHS
    'relative_months',
    'last_month',
    'this_month',

    # YEARS
    'last_year',
    'this_year'
]

DATE_FILTERS = namedtuple('DATE_FILTERS', [
    # ABSOLUTE
    'RANGE',
    'DAY',

    # HOURS
    'RELATIVE_HOURS',

    # DAYS
    'RELATIVE_DAYS',
    'YESTERDAY',
    'TODAY',

    # WEEKS
    'RELATIVE_WEEKS',
    'LAST_WEEK',
    'THIS_WEEK',

    # MONTHS
    'RELATIVE_MONTHS',
    'LAST_MONTH',
    'THIS_MONTH',

    # YEARS
    'LAST_YEAR',
    'THIS_YEAR'
])(*date_filters)

chart_types = [
    'bar',
    'column',
    'table',
    'area',
    'line',
    'pie',
    'scatter',
    'spline'
]

CHART_TYPES = namedtuple('CHART_TYPES', [
    'BAR',
    'COLUMN',
    'TABLE',
    'AREA',
    'LINE',
    'PIE',
    'SCATTER',
    'SPLINE'
])(*chart_types)

report_config = [
    'date_filters',
    'chart_types',
    'default_params'
]

REPORT_CONFIG = namedtuple('REPORT_CONFIG', [
    'DATE_FILTERS',
    'CHART_TYPES',
    'DEFAULT_PARAMS'
])(*report_config)


def get_mime_type_extension(mimetype):
    if mimetype == MIME_TYPES.PNG:
        return 'png'
    elif mimetype == MIME_TYPES.JPEG:
        return 'jpeg'
    elif mimetype == MIME_TYPES.GIF:
        return 'gif'
    elif mimetype == MIME_TYPES.PDF:
        return 'pdf'
    elif mimetype == MIME_TYPES.SVG:
        return 'svg'
    elif mimetype == MIME_TYPES.CSV:
        return 'csv'
    elif mimetype == MIME_TYPES.HTML:
        return 'html'


registered_reports = {}


def register_report(report_type, report_endpoint):
    registered_reports[report_type] = report_endpoint


def get_report_service(report_type):
    try:
        return get_resource_service(
            registered_reports[report_type]
        )
    except KeyError:
        return None


def is_highcharts_installed():
    try:
        check_call(['which', 'highcharts-export-server'], stdout=PIPE, stderr=PIPE)
        return True
    except Exception:
        return False


def get_cv_by_qcode(name, field=None):
    cvs = get_resource_service('vocabularies').find_one(req=None, _id=name)
    return {} if not cvs else {
        item.get('qcode'): item if field is None else item.get(field)
        for item in cvs.get('items') or []
        if item.get('is_active', True)
    }


def get_elastic_version():
    return app.data.elastic.es.info()['version']['number']


def get_weekstart_offset_hr():
    """Calculates the offset in hours for the configured start of the week

    This method is used with elasticsearch to provide an offset for histogram queries
    where the default buckets for week are based on the week starting on a Monday.

    :return: Week start offset in hours relative to Monday
    """
    offset = 0

    start_of_week = app.config.get('START_OF_WEEK') or 0
    if start_of_week == 0:
        offset -= 24
    elif start_of_week > 1:
        offset += (start_of_week - 1) * 24

    return offset


def get_utc_offset_in_minutes(utc_datetime):
    """Calculates the UTC Offset in minutes for the supplied datetime instance

    :param datetime utc_datetime: The date/time instance used to calculate utc offset
    :return: UTC Offset in minutes
    """
    timezone = pytz.timezone(app.config['DEFAULT_TIMEZONE'])
    return timezone.utcoffset(utc_datetime).total_seconds() / 60


def seconds_to_human_readable(seconds):
    """Converts seconds to a human readable format

    :param float seconds: Seconds to convert
    :return string: Human readable duration
    """
    if seconds >= 86400:
        if floor(seconds / 86400) == 1:
            return '1 day'

        return '{} days'.format(floor(seconds / 86400))
    elif seconds >= 3600:
        if floor(seconds / 3600) == 1:
            return '1 hour'

        return '{} hours'.format(floor(seconds / 3600))
    elif seconds >= 60:
        if floor(seconds / 60) == 1:
            return '1 minute'

        return '{} minutes'.format(floor(seconds / 60))
    elif floor(seconds) == 1:
        return '1 second'

    return '{} seconds'.format(floor(seconds))


def relative_to_absolute_datetime(value, format, now=None, offset=None):
    """Converts a relative datetime to an absolute datetime in the format provided

    Formats supported include:
    now
    now/[g]
    now-[o]
    now-[o]/[g]

    g = granularity (rounding the value down to the nearest value)
        - m = minute
        - h = hour
        - d = day (first hour/minute/second of the day)
        - w = week (first day of the week, using the system configured START_OF_WEEK value)
        - M = month (the first day of the month)
        - y = year (the first day of the year)

    o = offset from now suffixed by the granularity (see above for supported values)

    examples:
    now
    now/d
    now-1d
    now-1d/d

    :param string value: The relative datetime
    :param string format: The format used to convert the datetime instance back to a string
    :param datetime now: The date and time to use for relative calculations (defaults to now using DEFAULT_TIMEZONE)
    :param number offset: The utc offset in minutes (defaults to offset using the DEFAULT_TIMEZONE config)
    :return string: The absolute datetime in the format provided

    """

    try:
        values = re.search(r'^now(?P<offset>(-\d+[mhdwMy])?)(?P<granularity>(/[mhdwMy])?)$', value).groupdict()
    except AttributeError as e:
        logger.exception('Value {} is in incorrect relative format'.format(value))
        raise

    if now is None:
        now = utc_to_local(app.config.get('DEFAULT_TIMEZONE'), utcnow())

    if values.get('offset'):
        # Retrieve the offset value and granularity, then shift the datetime

        granularity = values['offset'][-1]
        offset = int(values['offset'][1:-1])

        if granularity == 'm':
            now = now - timedelta(minutes=offset)
        elif granularity == 'h':
            now = now - timedelta(hours=offset)
        elif granularity == 'd':
            now = now - timedelta(days=offset)
        elif granularity == 'w':
            now = now - timedelta(weeks=offset)
        elif granularity == 'M':
            now = now - relativedelta(months=offset)
        elif granularity == 'y':
            now = now - relativedelta(years=offset)

    if values.get('granularity'):
        # Round the value down using the granularity provided

        granularity = values['granularity'][1:]

        parts = None
        if granularity == 'm':
            parts = {'second': 0}
        elif granularity == 'h':
            parts = {
                'second': 0,
                'minute': 0
            }
        elif granularity == 'd':
            parts = {
                'second': 0,
                'minute': 0,
                'hour': 0
            }
        elif granularity == 'w':
            # Using START_OF_WEEK to calculate the number of days to shift for the beginning of the week
            # START_OF_WEEK
            #   0: Sunday
            #   6: Saturday

            isoweekday = now.isoweekday()
            if isoweekday == 7:
                isoweekday = 0

            start_of_week = app.config.get('START_OF_WEEK') or 0
            offset = 7 - start_of_week + isoweekday

            if offset < 7:
                now -= timedelta(days=offset)
            elif offset > 7:
                now -= timedelta(days=offset - 7)

            parts = {
                'second': 0,
                'minute': 0,
                'hour': 0
            }
        elif granularity == 'M':
            parts = {
                'second': 0,
                'minute': 0,
                'hour': 0,
                'day': 1
            }
        elif granularity == 'y':
            parts = {
                'second': 0,
                'minute': 0,
                'hour': 0,
                'day': 1,
                'month': 1
            }

        if parts:
            # Sets the second, minute, hour, day and/or month of the shifted value provided
            now = now.replace(**parts)

    return now.strftime(format)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.tests import TestCase
from analytics.reports import generate_report
from analytics.common import MIME_TYPES

from PIL import Image
from io import BytesIO


options = {
    "title": {"text": "Steep Chart"},
    "xAxis": {
        "categories": ["Jan", "Feb", "Mar"]
    },
    "series": [
        {"data": [29.9, 71.5, 106.4]}
    ]
}


class GenerateReportTestCase(TestCase):
    def test_generate_svg(self):
        with self.app.app_context():
            report = generate_report(options, mimetype=MIME_TYPES.SVG, base64=False)

            # Report is a buffer of bytes with size > 0
            self.assertTrue(type(report) == bytes)
            self.assertTrue(len(report) > 0)

            # The bytes buffer represents an xml/svg file
            self.assertTrue(report.startswith(b'<?xml'))
            self.assertTrue(report.find(b'<svg') > -1)

            # The title and categories are found in the svg data
            self.assertTrue(report.find(b'Steep Chart') > -1)
            self.assertTrue(report.find(b'Jan') > -1)
            self.assertTrue(report.find(b'Feb') > -1)
            self.assertTrue(report.find(b'Mar') > -1)

    def test_generate_png(self):
        with self.app.app_context():
            report = generate_report(options, mimetype=MIME_TYPES.PNG, base64=False, width='800')

            # Report is a buffer of bytes with size > 0
            self.assertTrue(type(report) == bytes)
            self.assertTrue(len(report) > 0)

            image = Image.open(BytesIO(report))

            # Valid image file
            try:
                image.verify()
            except Exception as e:
                self.failureException('Exception raised: {}'.format(e))

            # Image is PNG with a width of 800
            self.assertEqual(image.format, 'PNG')
            self.assertEqual(image.width, 800)

    def test_generate_jpg(self):
        with self.app.app_context():
            report = generate_report(options, mimetype=MIME_TYPES.JPEG, base64=False, width='1200')

            # Report is a buffer of bytes with size > 0
            self.assertTrue(type(report) == bytes)
            self.assertTrue(len(report) > 0)

            image = Image.open(BytesIO(report))

            # Valid image file
            try:
                image.verify()
            except Exception as e:
                self.failureException('Exception raised: {}'.format(e))

            # Image is JPEG with a width of 1200
            self.assertEqual(image.format, 'JPEG')
            self.assertEqual(image.width, 1200)

    def test_generate_pdf(self):
        with self.app.app_context():
            report = generate_report(options, mimetype=MIME_TYPES.PDF, base64=False, width='1200')

            # Report is a buffer of bytes with size > 0
            self.assertTrue(type(report) == bytes)
            self.assertTrue(len(report) > 0)

            # PDF header/footer signature
            self.assertTrue(report.startswith(b'%PDF-'))
            self.assertTrue(report.endswith(b'%%EOF\n'))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import get_resource_service
from superdesk.metadata.item import ASSOCIATIONS, CONTENT_TYPE
from superdesk.logging import logger

from analytics.stats.common import STAT_TYPE, OPERATION, FEATUREMEDIA_OPERATIONS
from analytics.stats.gen_archive_statistics import connect_stats_signals

from copy import deepcopy
from flask import current_app as app
from eve.utils import config


class FeaturemediaUpdates:
    def __init__(self, sender=None):
        self.rewrite_ids = set()

    def store_update_fields(self, sender, entry, update):
        if entry.get('operation') not in FEATUREMEDIA_OPERATIONS:
            return

        # Store the featuremedia updates for use during future iterations
        update.update({
            ASSOCIATIONS: (entry.get('update') or {}).get(ASSOCIATIONS) or None
        })

    def init(self, sender, stats):
        # Clear the featuremedia stats as we'll recalculate them here
        stats[STAT_TYPE.FEATUREMEDIA_UPDATES] = []

    def process(self, sender, entry, new_timeline, updates, update, stats):
        # Generating stats with PUBLISH_ASSOCIATED_ITEMS=True is currently not supported
        if app.config.get('PUBLISH_ASSOCIATED_ITEMS', False):
            return

        operation = entry.get('operation')

        updates.setdefault('_featuremedia', None)
        updates.setdefault('_published', False)

        item_type = updates.get('type') or 'text'

        if entry.get('_auto_generated', False):
            entry['update'] = update

        # If this is the initial publish operation and featuremedia has been added already
        # Then add this operation as the first in the featuremedia changes
        if operation == OPERATION.PUBLISH and updates.get('_featuremedia'):
            self._add_media_operation(
                entry,
                operation,
                new_timeline,
                updates,
                stats,
                OPERATION.PUBLISH,
                updates.get('_featuremedia')
            )
            # return
        elif item_type == CONTENT_TYPE.TEXT:
            # Calculate featuremedia stats
            associations = update.get(ASSOCIATIONS) or None

            media = None if associations is None else associations.get('featuremedia') or None

            # If there are no associations in this update, then media has not changed
            if associations is None:
                return

            if updates['_featuremedia'] is not None and \
                operation in [OPERATION.UPDATE, OPERATION.CORRECT] and \
                    (associations is None or media is None or not media.get('_id')):
                # If featuremedia is removed
                updates['_featuremedia'] = None
                self._add_media_operation(
                    entry,
                    operation,
                    new_timeline,
                    updates,
                    stats,
                    OPERATION.REMOVE_FEATUREMEDIA,
                    media
                )

                return

            # If there is no featuremedia, then don't calculate any timeline entries
            if media is None and updates['_featuremedia'] is None:
                return

            if updates['_featuremedia'] is None:
                updates['_featuremedia'] = media

                self._add_media_operation(
                    entry,
                    operation,
                    new_timeline,
                    updates,
                    stats,
                    OPERATION.ADD_FEATUREMEDIA,
                    media
                )
            elif media is not None:
                if media.get('_id') != updates['_featuremedia'].get('_id'):
                    updates['_featuremedia'] = media

                    self._add_media_operation(
                        entry,
                        operation,
                        new_timeline,
                        updates,
                        stats,
                        OPERATION.UPDATE_FEATUREMEDIA_IMAGE,
                        media
                    )
                elif self._renditions_changed(updates['_featuremedia'], media):
                    updates['_featuremedia'] = media

                    self._add_media_operation(
                        entry,
                        operation,
                        new_timeline,
                        updates,
                        stats,
                        OPERATION.UPDATE_FEATUREMEDIA_POI,
                        media
                    )
        elif item_type == CONTENT_TYPE.PICTURE:
            if updates['_featuremedia'] is None:
                updates['_featuremedia'] = deepcopy(update)
            elif self._renditions_changed(updates['_featuremedia'], update):
                updates['_featuremedia'] = update

                self._add_media_operation(
                    entry,
                    operation,
                    new_timeline,
                    updates,
                    stats,
                    OPERATION.CHANGE_IMAGE_POI,
                    update
                )

    def _add_media_operation(self, entry, operation, new_timeline, updates, stats, name, media=None):
        media_operation = deepcopy(entry)
        media_operation['_auto_generated'] = True
        media_operation['operation'] = name

        if media is not None:
            media_operation['update'] = {
                ASSOCIATIONS: {
                    'featuremedia': media
                }
            }

        if operation != name and not entry.get('_processed', False):  # and name != OPERATION.PUBLISH:
            new_timeline.append(media_operation)

        if updates['_published']:  # and not operation == OPERATION.PUBLISH:
            stats[STAT_TYPE.FEATUREMEDIA_UPDATES].append(media_operation)

    def _renditions_changed(self, original, updates):
        def poi_changed(original_poi, updated_poi):
            if original_poi.get('x') != updated_poi.get('x') or \
                    original_poi.get('y') != updated_poi.get('y'):
                return True
            return False

        if poi_changed(original.get('poi') or {}, updates.get('poi') or {}):
            return True

        original_renditions = original.get('renditions') or {}
        updated_renditions = updates.get('renditions') or {}

        if original_renditions.keys() != updated_renditions.keys():
            return True

        attribs = ['width', 'height', 'media', 'CropLeft', 'CropRight', 'CropTop', 'CropBottom']

        for key, original_rendition in original_renditions.items():
            updated_rendition = updated_renditions[key]

            if poi_changed(
                original_rendition.get('poi') or {},
                updated_rendition.get('poi') or {}
            ):
                return True

            for attrib in attribs:
                if original_rendition.get(attrib) != updated_rendition.get(attrib):
                    return True

        return False

    def complete(self, sender, stats, orig, updates):
        num_featuremedia_updates = len(stats.get(STAT_TYPE.FEATUREMEDIA_UPDATES) or [])
        if num_featuremedia_updates < 1:
            stats[STAT_TYPE.FEATUREMEDIA_UPDATES] = None
            updates['num_featuremedia_updates'] = 0
        else:
            updates['num_featuremedia_updates'] = num_featuremedia_updates

        if updates.get('rewrite_of'):
            self.rewrite_ids.add(updates.get('rewrite_of'))
            self.rewrite_ids.add(orig['_id'])

    def _get_featuremedia(self, entry):
        return None if entry is None else ((entry.get('update') or {}).get('associations') or {}).get('featuremedia')

    def finish(self, sender):
        service = get_resource_service('archive_statistics')

        query = {'query': {'bool': {'must': {'terms': {'_id': list(self.rewrite_ids)}}}}}
        docs = {
            doc['_id']: doc
            for doc in service.search(query)
        }

        def get_parent_id(doc):
            if not doc.get('rewrite_of'):
                return doc[config.ID_FIELD]
            elif doc['rewrite_of'] not in docs:
                # If the parent item was not part of this stats iteration
                # then load it now
                parent = service.find_one(req=None, _id=doc['rewrite_of'])

                if not parent:
                    # Stats entry for the parent item was not found
                    logger.warn('Failed to get parent item {}'.format(doc['rewrite_of']))
                    return None
                else:
                    docs[doc['rewrite_of']] = parent

            return get_parent_id(docs[doc['rewrite_of']])

        parent_docs = {}
        for doc_id in list(docs.keys()):
            doc = docs[doc_id]
            parent_id = get_parent_id(doc)

            if parent_id is None:
                # If we failed to find the parent id
                # Then the stats do not exist for the parent
                # So we ignore this entry
                continue

            updates = (doc.get('stats') or {}).get('featuremedia_updates') or []

            if len(updates) < 1:
                # This would indicate that there are no featuremedia updates recorded for this single item
                # find the previous featuremedia operation before the publish operation
                # and use that as the basis for the featuremedia updates
                last_entry = next((
                    entry for entry in reversed((doc.get('stats') or {}).get('timeline') or [])
                    if entry.get('operation') in FEATUREMEDIA_OPERATIONS
                ), None)

                if last_entry is not None:
                    updates = [last_entry]

            if len(updates) > 0:
                if parent_id not in parent_docs:
                    parent_docs[parent_id] = updates
                else:
                    parent_docs[parent_id] += updates

        for doc_id, stats in parent_docs.items():
            # get the archive_family stats item (if it exists)
            merged_id = '{}_family'.format(doc_id)
            original = service.find_one(req=None, _id=merged_id)
            is_new = False

            if not original:
                # If the archive_family stats item doesn't exist
                # then get the stats for the archive item itself
                is_new = True
                original = service.find_one(req=None, _id=doc_id) or {}

            sorted_entries = sorted(stats, key=lambda k: (k['operation_created'], k['history_id']))

            last_stat = sorted_entries[0]
            merged_stats = [sorted_entries[0]] if self._get_featuremedia(last_stat) else []

            for entry in sorted_entries[1:]:
                current_media = self._get_featuremedia(last_stat)
                next_media = self._get_featuremedia(entry)

                if current_media and not next_media:
                    # previous entry has featuremedia
                    # current entry does not
                    entry['operation'] = OPERATION.REMOVE_FEATUREMEDIA
                    merged_stats.append(entry)
                elif next_media and not current_media:
                    # previous entry does not have featuremedia
                    # current entry does
                    entry['operation'] = OPERATION.ADD_FEATUREMEDIA
                    merged_stats.append(entry)
                elif current_media.get('_id') != next_media.get('_id'):
                    # previous entry has different featuremedia id to the current entry
                    entry['operation'] = OPERATION.UPDATE_FEATUREMEDIA_IMAGE
                    merged_stats.append(entry)
                elif self._renditions_changed(current_media, next_media):
                    # previous entry has different poi
                    entry['operation'] = OPERATION.UPDATE_FEATUREMEDIA_POI
                    merged_stats.append(entry)

                last_stat = entry

            updates = deepcopy(original)
            updates['stats_type'] = 'archive_family'

            num_featuremedia_updates = len(merged_stats)
            if num_featuremedia_updates < 1:
                merged_stats = None
                updates['num_featuremedia_updates'] = 0
            else:
                updates['num_featuremedia_updates'] = num_featuremedia_updates

            if 'stats' not in updates:
                updates['stats'] = {}

            updates['stats']['featuremedia_updates'] = merged_stats

            if is_new:
                updates['_id'] = merged_id
                service.post([updates])
            else:
                service.patch(
                    merged_id,
                    {
                        'stats': updates['stats'],
                        'num_featuremedia_updates': updates['num_featuremedia_updates']
                    }
                )


featuremedia_updates = FeaturemediaUpdates()

connect_stats_signals(
    on_start=featuremedia_updates.__init__,
    on_generate=featuremedia_updates.store_update_fields,
    on_init_timeline=featuremedia_updates.init,
    on_process=featuremedia_updates.process,
    on_complete=featuremedia_updates.complete,
    on_finish=featuremedia_updates.finish
)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from .user_acitivity_report import UserActivityReportResource, UserActivityReportService
from analytics.common import register_report


def init_app(app):
    # Don't register this endpoint if archive stats aren't being generated
    if not app.config.get('ANALYTICS_ENABLE_ARCHIVE_STATS', False):
        return

    endpoint_name = 'user_activity_report'
    service = UserActivityReportService(endpoint_name, backend=superdesk.get_backend())
    UserActivityReportResource(endpoint_name, app=app, service=service)

    register_report('user_activity_report', 'user_activity_report')

    superdesk.privilege(
        name='user_activity_report',
        label='Analytics - User Activity Reports',
        description='User can view User Activity Reports'
    )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import json, get_resource_service
from superdesk.services import BaseService
from superdesk.resource import Resource
from superdesk.notification import push_notification
from superdesk.errors import SuperdeskApiError
from superdesk.users.services import current_user_has_privilege

from apps.auth import get_user_id
from apps.archive.common import get_user, get_auth

from eve.utils import ParsedRequest

from analytics.common import get_report_service


class SavedReportsResource(Resource):
    endpoint_name = resource_title = 'saved_reports'
    schema = {
        'name': {
            'type': 'string',
            'required': True,
            'minlength': 1
        },
        'description': {
            'type': 'string'
        },
        'report': {
            'type': 'string',
            'required': True
        },
        'params': {
            'type': 'dict',
            'required': True
        },
        'user': Resource.rel('users', nullable=True),
        'is_global': {
            'type': 'boolean',
            'default': False
        },
        'translations': {
            'type': 'dict',
            'required': False
        }
    }

    url = 'saved_reports'
    item_methods = ['GET', 'PATCH', 'DELETE']
    resource_methods = ['GET', 'POST']

    privileges = {
        'POST': 'saved_reports',
        'PATCH': 'saved_reports',
        'DELETE': 'saved_reports'
    }


class SavedReportsService(BaseService):
    def on_create(self, docs):
        for doc in docs:
            self._validate_on_create(doc)
            doc['user'] = get_user_id(required=True)
        super().on_create(docs)

    def on_created(self, docs):
        for doc in docs:
            self._push_notification(doc, 'create')

    def on_update(self, updates, original):
        """Runs on update

        Checks if the request owner and the saved search owner are the same person
        If not then the request owner should have global saved reports privilege
        """
        self._validate_on_update(updates, original)
        self._null_updates_from_original(updates, original)
        super().on_update(updates, original)

    def on_updated(self, updates, original):
        self._push_notification(original, 'update')

    def on_delete(self, doc):
        self._validate_on_delete(doc)

    def on_deleted(self, doc):
        self._push_notification(doc, 'delete')

    def get(self, req, lookup):
        """
        Overriding to pass user as search parameter
        """
        session_user = str(get_user_id(required=True))

        if not req:
            req = ParsedRequest()

        where = json.loads(req.where) if req.where else {}

        if lookup:
            where.update(lookup)

        if '$or' not in where:
            where['$or'] = []

        # Restrict the saved reports to either global or owned by current user
        where['$or'].extend([
            {'is_global': True},
            {'user': session_user}
        ])

        req.where = json.dumps(where)

        return super().get(req, lookup=None)

    @staticmethod
    def _push_notification(doc, operation):
        push_notification(
            'savedreports:update',
            report_type=doc['report'],
            operation=operation,
            report_id=str(doc.get('_id')),
            user_id=str(get_user_id()),
            session_id=str(get_auth().get('_id'))
        )

    @staticmethod
    def _validate_on_create(doc):
        """
        User can only create global report if they have 'global_saved_reports' permission
        """
        if doc.get('is_global', False) and not current_user_has_privilege('global_saved_reports'):
            raise SuperdeskApiError.forbiddenError('Unauthorized to create global report.')

    @staticmethod
    def _validate_ownership(doc):
        """
        Validate saved reports on update/delete

        User can only update/delete their own reports, or global reports if they the
        'global_saved_reports' have permission
        """
        session_user = get_user(required=True)
        if str(session_user['_id']) != str(doc.get('user', '')):
            if not doc.get('is_global', False):
                raise SuperdeskApiError.forbiddenError(
                    'Unauthorized to modify other user\'s local report.'
                )
            elif not current_user_has_privilege('global_saved_reports'):
                raise SuperdeskApiError.forbiddenError('Unauthorized to modify global report.')

    def _validate_on_update(self, updates, original):
        self._validate_ownership(original)

        scheduled_service = get_resource_service('scheduled_reports')
        schedules = scheduled_service.get(
            req=None,
            lookup={'saved_report': original['_id']}
        )

        if schedules.count() > 0:
            if original.get('is_global') and not updates.get('is_global'):
                raise SuperdeskApiError.badRequestError(
                    'Cannot remove global flag as schedule(s) are attached'
                )

    def _validate_on_delete(self, doc):
        self._validate_ownership(doc)

        scheduled_service = get_resource_service('scheduled_reports')
        schedules = scheduled_service.get(
            req=None,
            lookup={'saved_report': doc['_id']}
        )

        if schedules.count() > 0:
            raise SuperdeskApiError.badRequestError(
                'Cannot delete saved report as schedule(s) are attached'
            )

    def get_aggregations(self, req, **lookup):
        saved_report = self.find_one(req=req, **lookup)
        if not saved_report:
            raise SuperdeskApiError.notFoundError('Saved report not found')

        report_service = get_report_service(saved_report.get('report'))
        if report_service is None:
            raise SuperdeskApiError.badRequestError('Invalid report type')

        if not getattr(report_service, 'generate_request_from_saved_report'):
            raise SuperdeskApiError.internalError(
                'Cannot generate aggregations from this saved report'
            )

        request = report_service.generate_request_from_saved_report(saved_report)
        aggregations = list(report_service.get(request, lookup=None))[0]

        return saved_report, aggregations

    def _null_updates_from_original(self, updates, original):
        """Null values that are not in the updated report params"""

        self._null_values(updates, original, 'translations')

        if 'params' in updates:
            self._null_values(updates['params'], original['params'], 'must')
            self._null_values(updates['params'], original['params'], 'must_not')
            self._null_values(updates['params'], original['params'], 'chart')
            self._null_values(updates['params'], original['params'], 'dates')
            self._null_values(updates['params'], original['params'], 'aggs')
            self._null_values(updates['params'], original['params'], 'repos')

    def _null_values(self, updates, original, field):
        """Sets values to None that are in original but not in updates"""

        if field not in updates:
            return

        for key in (original or {}).get(field) or {}:
            if key not in updates[field]:
                updates[field][key] = None



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource

from analytics.base_report import BaseReportService
from analytics.chart_config import ChartConfig


class ContentPublishingReportResource(Resource):
    """Content Publishing Report schema
    """

    item_methods = ['GET']
    resource_methods = ['GET']
    privileges = {'GET': 'content_publishing_report'}


class ContentPublishingReportService(BaseReportService):
    repos = ['published', 'archived']

    aggregations = {
        'source': {
            'terms': {
                'field': 'source',
                'size': 0,
            }
        }
    }

    @staticmethod
    def _get_aggregation_query(agg):
        include = agg.get('filter') or 'all'

        query = {
            'terms': {
                'field': agg.get('field'),
                'size': agg.get('size') or 0
            }
        }

        if include != 'all':
            query['terms']['include'] = include
            query['terms']['min_doc_count'] = 0

        return query

    def get_aggregations(self, params, args):
        aggs = params.get('aggs') or {}

        if not aggs or not (aggs.get('group') or {}).get('field'):
            return ContentPublishingReportService.aggregations

        aggregations = {
            'parent': self._get_aggregation_query(aggs['group'])
        }

        if aggs.get('subgroup'):
            aggregations['parent']['aggs'] = {
                'child': self._get_aggregation_query(aggs['subgroup'])
            }

        return aggregations

    def generate_report(self, docs, args):
        """Returns the publishing statistics

        :param docs: document used for generating the statistics
        :return dict: report
        """
        agg_buckets = self.get_aggregation_buckets(getattr(docs, 'hits'), ['parent'])

        report = {'groups': {}}

        has_children = 'field' in ((args.get('aggs') or {}).get('subgroup') or {})

        if has_children:
            report['subgroups'] = {}

        for parent in agg_buckets.get('parent') or []:
            parent_key = parent.get('key')

            if not parent_key:
                continue

            if not has_children:
                report['groups'][parent_key] = parent.get('doc_count') or 0
                continue

            report['groups'][parent_key] = {}

            for child in (parent.get('child') or {}).get('buckets') or []:
                child_key = child.get('key')

                if not child_key:
                    continue

                if child_key not in report['subgroups']:
                    report['subgroups'][child_key] = 0

                doc_count = child.get('doc_count') or 0
                report['groups'][parent_key][child_key] = doc_count
                report['subgroups'][child_key] += doc_count

        return report

    def generate_highcharts_config(self, docs, args):
        params = args.get('params') or {}
        aggs = args.get('aggs') or {}
        group = aggs.get('group') or {}
        subgroup = aggs.get('subgroup') or {}
        translations = args.get('translations') or {}

        chart = params.get('chart') or {}
        chart_type = chart.get('type') or 'bar'

        report = self.generate_report(docs, args)

        chart_config = ChartConfig('content_publishing', chart_type)

        chart_config.add_source(group.get('field'), report.get('groups'))

        if report.get('subgroups'):
            chart_config.add_source(subgroup.get('field'), report['subgroups'])

        def gen_title():
            if chart.get('title'):
                return chart['title']

            group_type = group.get('field')
            group_title = chart_config.get_source_name(group_type)

            if subgroup.get('field'):
                subgroup_type = subgroup.get('field')
                subgroup_title = chart_config.get_source_name(subgroup_type)

                return 'Published Stories per {} with {} breakdown'.format(
                    group_title,
                    subgroup_title
                )

            return 'Published Stories per {}'.format(group_title)

        def gen_subtitle():
            return ChartConfig.gen_subtitle_for_dates(params)

        chart_config.get_title = gen_title
        chart_config.get_subtitle = gen_subtitle
        chart_config.sort_order = chart.get('sort_order') or 'desc'

        chart_config.translations = translations

        report['highcharts'] = [chart_config.gen_config()]

        return report



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


class Series:
    """Class instance for adding a series to an axis"""

    def __init__(self, axis):
        """Sets the initial data for the series

        :param analytics.chart_config.SDChart.Axis axis: The parent Axis instance
        """

        self.axis = axis
        self.chart = self.axis.chart

        self.type = None
        self.data = None
        self.field = None
        self.name = None
        self.stack = None
        self.stack_type = None

    def set_options(self, **kwargs):
        """Sets the options for the series

        kwargs:
            - ``type (str=self.axis.default_chart_type)``: The chart type
            - ``data (dict, list)``: The data to add to the series
            - ``field (str)``: The field type for the data
            - ``name (str)``: The field name for the data
            - ``stack (int)``: The stack number
            - ``stack_type (str)``: The type of stacking to perform
        """
        for key, value in kwargs.items():
            self.__dict__[key] = value

        return self

    def get_data(self):
        """Returns the data for this series"""

        if self.data is None:
            return None
        elif isinstance(self.data, list):
            return self.data
        elif isinstance(self.data, dict):
            if self.axis.categories is not None:
                return [
                    self.data.get(source) or 0
                    for source in self.axis.categories
                ]

            return [
                item.get(source) or 0
                for item, source in self.data.items()
            ]

        return None

    def get_name(self):
        """Returns the name of the field type"""

        if self.field is None:
            return self.name
        elif self.name is not None:
            name = (self.chart.get_translation_names(self.field) or {}).get(self.name) or self.name
        else:
            name = self.chart.get_translation_title(self.field) or self.name

        return str(name or self.field)

    def gen_config(self, config):
        """Sets the series config for the axis"""

        series = {
            'xAxis': self.axis.index,
            'type': self.type or self.axis.default_chart_type or 'bar'
        }
        name = self.get_name()
        data = self.get_data()

        if name is not None:
            series['name'] = name

        if data is not None:
            series['data'] = data

        if self.stack is not None:
            series['stacking'] = self.stack_type or 'normal'
            series['stack'] = self.stack

        if self.axis.point_start is not None:
            series['pointStart'] = self.axis.point_start

        if self.axis.point_interval is not None:
            series['pointInterval'] = self.axis.point_interval

        return series



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource

from analytics.stats.stats_report_service import StatsReportService
from analytics.common import REPORT_CONFIG, CHART_TYPES, DATE_FILTERS

from eve_elastic.elastic import parse_date


class UserActivityReportResource(Resource):
    """User Activity Report schema"""

    item_methods = ['GET']
    resource_methods = ['GET']
    privileges = {'GET': 'user_activity_report'}


class UserActivityReportService(StatsReportService):
    date_filter_field = 'versioncreated'

    defaultConfig = {
        REPORT_CONFIG.DATE_FILTERS: {
            # Day filter is the only type of filter available to 'UserActivity' report
            # The 'report_configs' endpoint will omit all others
            DATE_FILTERS.DAY: {'enabled': True},
        },
        REPORT_CONFIG.CHART_TYPES: {
            # Table is the only supported chart type for this report
            # The 'report_configs' endpoint will omit all others
            CHART_TYPES.TABLE: {'enabled': True}
        },
        REPORT_CONFIG.DEFAULT_PARAMS: {
            'chart': {'type': CHART_TYPES.TABLE}
        }
    }

    def get_request_aggregations(self, params, args):
        """Disable generating aggregations"""
        return None

    def generate_report(self, docs, args):
        report = {
            'items': [],
            'min': 0,
            'max': 0
        }

        for doc in docs:
            stats = doc.get('stats') or {}

            def calc_timestamp(entry):
                entry['operation_timestamp'] = parse_date(
                    entry.get('operation_created')
                ).timestamp()

                return entry

            timeline = [
                calc_timestamp(entry)
                for entry in stats.get('timeline') or []
            ]

            entry = {
                '_id': doc.get('_id'),
                'timeline': timeline,
                'slugline': doc.get('slugline') or '',
                'headline': doc.get('headline') or '',
                'activity': [],
            }

            locks = [
                lock for lock in timeline
                if lock.get('operation') in ['item_lock', 'item_unlock']
            ]

            user_id = ((args.get('params') or {}).get('must') or {}).get('user_locks')
            current_lock = None
            for lock in locks:
                # If this lock is not for the user provided in the argument filter
                # Then skip processing this lock entry
                if user_id != (lock.get('task') or {}).get('user'):
                    continue

                operation = lock.get('operation')
                created = lock.get('operation_timestamp')
                if current_lock is None:
                    if operation == 'item_lock':
                        current_lock = [created]
                elif operation == 'item_unlock':
                    current_lock.append(created)
                    entry['activity'].append(current_lock)

                    if report['min'] == 0 or current_lock[0] < report['min']:
                        report['min'] = current_lock[0]

                    if report['max'] == 0 or current_lock[1] > report['max']:
                        report['max'] = current_lock[1]

                    current_lock = None

            if len(entry['activity']) > 0:
                report['items'].append(entry)

        if len(report['items']) < 1:
            return {}

        return report



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from superdesk.tests import TestCase

from analytics.common import get_weekstart_offset_hr, get_utc_offset_in_minutes,\
    seconds_to_human_readable, relative_to_absolute_datetime

from datetime import datetime, timedelta


class CommonTestCase(TestCase):
    def test_weekstart_offset_hr(self):
        self.app.config['START_OF_WEEK'] = 0
        self.assertEqual(get_weekstart_offset_hr(), -24)

        self.app.config['START_OF_WEEK'] = 1
        self.assertEqual(get_weekstart_offset_hr(), 0)

        self.app.config['START_OF_WEEK'] = 2
        self.assertEqual(get_weekstart_offset_hr(), 24)

        self.app.config['START_OF_WEEK'] = 3
        self.assertEqual(get_weekstart_offset_hr(), 48)

        self.app.config['START_OF_WEEK'] = 4
        self.assertEqual(get_weekstart_offset_hr(), 72)

        self.app.config['START_OF_WEEK'] = 5
        self.assertEqual(get_weekstart_offset_hr(), 96)

        self.app.config['START_OF_WEEK'] = 6
        self.assertEqual(get_weekstart_offset_hr(), 120)

    def test_utc_offset_in_minutes(self):
        self.app.config['DEFAULT_TIMEZONE'] = 'Australia/Sydney'
        self.assertEqual(
            get_utc_offset_in_minutes(datetime(2018, 10, 1)),
            600
        )

        self.assertEqual(
            get_utc_offset_in_minutes(datetime(2018, 10, 10)),
            660
        )

    def test_seconds_to_human_readable(self):
        # Seconds
        self.assertEqual(seconds_to_human_readable(1), '1 second')
        self.assertEqual(seconds_to_human_readable(1.5), '1 second')
        self.assertEqual(seconds_to_human_readable(10), '10 seconds')

        # Minutes
        self.assertEqual(seconds_to_human_readable(60), '1 minute')
        self.assertEqual(seconds_to_human_readable(90), '1 minute')
        self.assertEqual(seconds_to_human_readable(120), '2 minutes')
        self.assertEqual(seconds_to_human_readable(150), '2 minutes')

        # Hours
        self.assertEqual(seconds_to_human_readable(3600), '1 hour')
        self.assertEqual(seconds_to_human_readable(5400), '1 hour')
        self.assertEqual(seconds_to_human_readable(7200), '2 hours')
        self.assertEqual(seconds_to_human_readable(9000), '2 hours')

        # Days
        self.assertEqual(seconds_to_human_readable(86400), '1 day')
        self.assertEqual(seconds_to_human_readable(129600), '1 day')
        self.assertEqual(seconds_to_human_readable(172800), '2 days')
        self.assertEqual(seconds_to_human_readable(216000), '2 days')

    def test_relative_to_absolute_datetime(self):
        self.app.config.update({
            'DEFAULT_TIMEZONE': 'Australia/Sydney',
            'START_OF_WEEK': 0  # Sunday
        })
        fm = '%Y-%m-%dT%H:%M:%S'
        dt = datetime(2019, 10, 25, 13, 25, 52)

        # Test without any optional parameters
        self.assertEqual(relative_to_absolute_datetime('now', fm, dt), '2019-10-25T13:25:52')

        # Test rounding down
        self.assertEqual(relative_to_absolute_datetime('now/m', fm, dt), '2019-10-25T13:25:00')
        self.assertEqual(relative_to_absolute_datetime('now/h', fm, dt), '2019-10-25T13:00:00')
        self.assertEqual(relative_to_absolute_datetime('now/d', fm, dt), '2019-10-25T00:00:00')
        self.assertEqual(relative_to_absolute_datetime('now/M', fm, dt), '2019-10-01T00:00:00')
        self.assertEqual(relative_to_absolute_datetime('now/y', fm, dt), '2019-01-01T00:00:00')

        # Test subtraction
        self.assertEqual(relative_to_absolute_datetime('now-1m', fm, dt), '2019-10-25T13:24:52')
        self.assertEqual(relative_to_absolute_datetime('now-1h', fm, dt), '2019-10-25T12:25:52')
        self.assertEqual(relative_to_absolute_datetime('now-1d', fm, dt), '2019-10-24T13:25:52')
        self.assertEqual(relative_to_absolute_datetime('now-1w', fm, dt), '2019-10-18T13:25:52')
        self.assertEqual(relative_to_absolute_datetime('now-1M', fm, dt), '2019-09-25T13:25:52')
        self.assertEqual(relative_to_absolute_datetime('now-1y', fm, dt), '2018-10-25T13:25:52')

        self.assertEqual(relative_to_absolute_datetime('now-26m', fm, dt), '2019-10-25T12:59:52')
        self.assertEqual(relative_to_absolute_datetime('now-26h', fm, dt), '2019-10-24T11:25:52')
        self.assertEqual(relative_to_absolute_datetime('now-26d', fm, dt), '2019-09-29T13:25:52')
        self.assertEqual(relative_to_absolute_datetime('now-26w', fm, dt), '2019-04-26T13:25:52')
        self.assertEqual(relative_to_absolute_datetime('now-26M', fm, dt), '2017-08-25T13:25:52')
        self.assertEqual(relative_to_absolute_datetime('now-26y', fm, dt), '1993-10-25T13:25:52')

        # Test subtraction and rounding down
        self.assertEqual(relative_to_absolute_datetime('now-1m/m', fm, dt), '2019-10-25T13:24:00')
        self.assertEqual(relative_to_absolute_datetime('now-1m/h', fm, dt), '2019-10-25T13:00:00')
        self.assertEqual(relative_to_absolute_datetime('now-1m/d', fm, dt), '2019-10-25T00:00:00')
        self.assertEqual(relative_to_absolute_datetime('now-1m/w', fm, dt), '2019-10-20T00:00:00')
        self.assertEqual(relative_to_absolute_datetime('now-1m/M', fm, dt), '2019-10-01T00:00:00')
        self.assertEqual(relative_to_absolute_datetime('now-1m/y', fm, dt), '2019-01-01T00:00:00')

        self.assertEqual(relative_to_absolute_datetime('now-26m/m', fm, dt), '2019-10-25T12:59:00')
        self.assertEqual(relative_to_absolute_datetime('now-26m/h', fm, dt), '2019-10-25T12:00:00')
        self.assertEqual(relative_to_absolute_datetime('now-26m/d', fm, dt), '2019-10-25T00:00:00')
        self.assertEqual(relative_to_absolute_datetime('now-26m/w', fm, dt), '2019-10-20T00:00:00')
        self.assertEqual(relative_to_absolute_datetime('now-26m/M', fm, dt), '2019-10-01T00:00:00')
        self.assertEqual(relative_to_absolute_datetime('now-26m/y', fm, dt), '2019-01-01T00:00:00')

    def test_relative_to_absolute_datetime_week_granularity(self):
        """Starting on Sunday 2019-10-20, test shifting the week using different START_OF_WEEK values"""

        results = {
            'Sunday': {
                'start_of_week': 0,
                'results': [20, 20, 20, 20, 20, 20, 20]
            },
            'Monday': {
                'start_of_week': 1,
                'results': [14, 21, 21, 21, 21, 21, 21]
            },
            'Tuesday': {
                'start_of_week': 2,
                'results': [15, 15, 22, 22, 22, 22, 22]
            },
            'Wednesday': {
                'start_of_week': 3,
                'results': [16, 16, 16, 23, 23, 23, 23]
            },
            'Thursday': {
                'start_of_week': 4,
                'results': [17, 17, 17, 17, 24, 24, 24]
            },
            'Friday': {
                'start_of_week': 5,
                'results': [18, 18, 18, 18, 18, 25, 25]
            },
            'Saturday': {
                'start_of_week': 6,
                'results': [19, 19, 19, 19, 19, 19, 26]
            },
        }

        self.app.config['DEFAULT_TIMEZONE'] = 'Australia/Sydney'
        fm = '%Y-%m-%dT%H:%M:%S'

        for day, data in results.items():
            self.app.config['START_OF_WEEK'] = data['start_of_week']

            for i in range(0, 7):
                self.assertEqual(
                    relative_to_absolute_datetime(
                        'now/w',
                        fm,
                        datetime(2019, 10, 20 + i, 13, 25, 52)
                    ),
                    '2019-10-{}T00:00:00'.format(data['results'][i]),
                    'Expected: {} {}'.format(day, i)
                )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from .series import Series


class Axis:
    """Class instance for adding an Axis with series of data to a chart"""

    def __init__(self, chart):
        """Sets the initial data for the axis

        :param analytics.chart_config.SDChart.Chart chart: The parent Chart instance
        """

        self.chart = chart
        self.type = 'linear'
        self.index = 0
        self.allow_decimals = False
        self.series = []

        self.default_chart_type = None
        self.categories = None
        self.category_field = None
        self.point_start = None
        self.point_interval = None
        self.stack_labels = None
        self.y_title = None
        self.x_title = None

    def set_options(self, **kwargs):
        """Sets the options for the axis

        kwargs:
            - ``type (str='linear')``: The Axis type, i.e. linear, logarithmic, datetime or category
            - ``default_chart_type (str)``: The default chart type for child series
            - ``index (int=0)``: The Axis index assigned by parent chart
            - ``category_field (str)``: Field used to translate category names
            - ``categories (list)``:  The list of categories
            - ``allow_decimals (bool=False)``: Use whole number on this axis
            - ``point_start (int)``: The starting point
            - ``point_interval (int)``: The intervals between points
            - ``stack_labels (bool)``: If true, then place labels at the top of stack
            - ``y_title (str)``: The title used on the y-axis
            - ``x_title (str)``: The title used on the x-axis
        """
        for key, value in kwargs.items():
            self.__dict__[key] = value

        return self

    def add_series(self):
        """Add a new data series to this axis sources"""

        series = Series(self)
        self.series.append(series)
        return series

    def get_categories(self):
        """Returns translated category field names"""

        if self.categories is not None and self.category_field is not None:
            names = self.chart.get_translation_names(self.category_field)

            return [
                names.get(category) or category
                for category in self.categories
            ]

        return self.categories

    def gen_x_axis_config(self, config):
        """Generate the x-axis config"""

        axis_config = {'type': self.type}

        if self.categories is not None:
            axis_config['categories'] = self.get_categories()

        if self.allow_decimals is not None:
            axis_config['allowDecimals'] = self.allow_decimals

        if self.chart.start_of_week is not None:
            axis_config['startOfWeek'] = self.chart.start_of_week

        if self.x_title is not None:
            axis_config['title'] = {'text': self.x_title}

        return axis_config

    def gen_y_axis_config(self, config):
        """Generate the y-axis config"""

        axis_config = {}

        if self.allow_decimals is not None:
            axis_config['allowDecimals'] = self.allow_decimals

        if self.stack_labels is not None:
            axis_config['stackLabels'] = {'enabled': self.stack_labels}

        if self.y_title is not None:
            axis_config['title'] = {'text': self.y_title}

        return axis_config

    def gen_config(self, config):
        """Generate the config"""

        if not config.get('xAxis'):
            config['xAxis'] = []

        config['xAxis'].append(self.gen_x_axis_config(config))

        if not config.get('yAxis'):
            config['yAxis'] = []

        config['yAxis'].append(self.gen_y_axis_config(config))

        if not config.get('series'):
            config['series'] = []

        for series in self.series:
            config['series'].append(series.gen_config(config))

        return config



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import requests
from flask import current_app as app
import csv
from io import StringIO

from superdesk.errors import SuperdeskApiError
from analytics.common import MIME_TYPES

logger = logging.getLogger(__name__)


def generate_report(
        options,
        mimetype=MIME_TYPES.PNG,
        base64=True,
        scale=1,
        width=None,
        no_download=True
):
    if not isinstance(options, dict):
        raise SuperdeskApiError.badRequestError('Provided options must be a dictionary')

    if 'series' not in options and 'rows' not in options:
        raise SuperdeskApiError.badRequestError('Series data not provided')

    if mimetype in [
        MIME_TYPES.PNG,
        MIME_TYPES.JPEG,
        MIME_TYPES.GIF,
        MIME_TYPES.PDF,
        MIME_TYPES.SVG
    ]:
        return generate_from_highcharts(options, mimetype, base64, scale, width, no_download)
    elif mimetype == MIME_TYPES.CSV:
        return generate_csv(options)
    elif mimetype == MIME_TYPES.HTML:
        return generate_html(options)

    raise SuperdeskApiError.badRequestError("Unsupported mimetype '{}'".format(mimetype))


def generate_from_highcharts(
        options,
        mimetype=MIME_TYPES.PNG,
        base64=True,
        scale=1,
        width=None,
        no_download=True
):
    try:
        host = app.config.get('HIGHCHARTS_SERVER_HOST', 'localhost')
        port = app.config.get('HIGHCHARTS_SERVER_PORT', '6060')
    except RuntimeError:
        # This can happen when working outside of the Flask context
        # So default to host=localhost, port=6060
        host = 'localhost'
        port = '6060'

    url = 'http://{}:{}'.format(host, port)
    headers = {'Content-Type': 'application/json'}

    # Set the width size of the image to generate
    if 'exporting' not in options:
        options['exporting'] = {}
    options['exporting']['sourceWidth'] = width

    payload = {
        'options': options,
        'type': mimetype,
        'b64': base64,
        'scale': scale,
        'noDownload': no_download
    }

    try:
        response = requests.post(url, headers=headers, json=payload)
    except requests.exceptions.ConnectionError as e:
        raise SuperdeskApiError.internalError('Socket connection error: {}'.format(e))

    try:
        response.raise_for_status()
    except Exception as e:
        logger.exception(e)
        raise SuperdeskApiError.internalError(e)

    return response.content


def generate_csv(options):
    csv_rows = options.get('csv') or []
    csv_file = StringIO()
    csv_writer = csv.writer(csv_file)

    csv_writer.writerows(csv_rows)

    return csv_file.getvalue().encode('UTF-8')


def generate_html(options):
    rows = options.get('rows') or []
    headers = options.get('headers') or []
    title = options.get('title') or ''

    if len(rows) < 1:
        return '''<div><h3>{}</h3></div>'''.format(title)

    thead = '<tr><th>{}</th></tr>'.format('</th><th>'.join(headers))
    tbody = ''
    for row in rows:
        tbody += '<tr><td>{}</td></tr>'.format(
            '</td><td>'.join([str(td) for td in row])
        )

    return '''
<div>
    <h3>{}</h3>
    <table border=1 style="width: 100%;">
        <thead>
            {}
        </thead>
        <tbody>
            {}
        </tbody>
    </table>
<div>'''.format(title, thead, tbody)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.tests.steps import when, then, assert_200, assert_equal, get_json_data, json,\
    fail_and_print_body, apply_placeholders, json_match

from analytics.stats.gen_archive_statistics import GenArchiveStatistics


@then('we get {total_count} charts')
def step_impl_then_get_charts(context, total_count):
    assert_200(context.response)
    data = get_json_data(context.response)
    int_count = int(total_count)
    report = (data.get('_items') or [{}])[0]
    num_reports = len(report.get('highcharts'))

    assert int_count == num_reports, 'Number of charts not equal. {} != {}'.format(
        int_count,
        num_reports
    )

    if context.text:
        try:
            response_data = json.loads(context.response.get_data())
        except Exception:
            fail_and_print_body(context.response, 'response is not valid json')
            return

        report = (response_data.get('_items') or [{}])[0]
        context_data = json.loads(apply_placeholders(context, context.text))

        chart_index = 0
        for context_chart in context_data:
            response_chart = report.get('highcharts')[chart_index]

            for key, value in context_chart.items():
                if isinstance(value, dict):
                    for subkey, subvalue in value.items():
                        assert response_chart[key][subkey] == subvalue,\
                            'chart[{}][{}][{}] {} != {}'.format(
                            chart_index,
                            key,
                            subkey,
                            subvalue,
                            response_chart[key][subkey]
                        )
                else:
                    assert response_chart[key] == value, 'chart[{}][{}] {} != {}'.format(
                        chart_index,
                        key,
                        value,
                        response_chart[key]
                    )

            chart_index += 1
        return response_data


@when('we generate stats from archive history')
def step_impl_when_generate_stats_from_archive_history(context):
    with context.app.app_context():
        GenArchiveStatistics().run()


@then('we get "{report_id}" config')
def step_impl_then_we_get_config(context, report_id):
    assert_200(context.response)

    if not context.text:
        return

    data = get_json_data(context.response)

    config = next((
        c for c in (data.get('_items') or [])
        if c.get('_id') == report_id
    ), None)

    expected_config = json.loads(apply_placeholders(context, context.text))
    assert_equal(json_match(expected_config, config), True)


@then('we get stats')
def step_impl_then_get_stats_for_item(context):
    assert_200(context.response)

    if context.text:
        try:
            response_data = json.loads(context.response.get_data())
        except Exception:
            fail_and_print_body(context.response, 'response is not valid json')
            return

        stats = response_data.get('stats') or {}
        context_stats = json.loads(apply_placeholders(context, context.text))

        # parent stat entries (i.e. timeline, desk_transitions, featuremedia_updates)
        for stat_type, stat_entries in context_stats.items():
            assert stat_type in stats.keys(), 'stats.{} does not exist'.format(stat_type)

            if stat_entries is None:
                assert stats[stat_type] is None, 'stats.{} is not empty'.format(
                    stat_type
                )
                continue
            elif stats[stat_type] is None:
                assert stat_entries == stats[stat_type], 'stats.{} {} != {}'.format(
                    stat_type,
                    stats[stat_type],
                    stat_entries
                )

            assert len(stat_entries) == len(stats[stat_type]),\
                'stats.{}. len {} != {}.\nStats={}'.format(
                stat_type,
                len(stat_entries),
                len(stats[stat_type]),
                stats[stat_type]
            )

            stat_index = 0
            for stat_entry in stat_entries:
                expected_stats = stats[stat_type][stat_index]

                for key, value in stat_entry.items():
                    assert key in expected_stats.keys(), 'stats.{}[{}] key "{}" not found'.format(
                        stat_type,
                        stat_index,
                        key
                    )

                    if isinstance(value, dict):
                        for subkey, subvalue in value.items():
                            assert subkey in expected_stats[key].keys(),\
                                'stats.{}[{}][{}] key "{}" not found.\nEntry={}'.format(
                                stat_type,
                                stat_index,
                                key,
                                subkey,
                                expected_stats[key]
                            )
                            assert json_match(subvalue, expected_stats[key][subkey]),\
                                'stats.{}[{}].{}.{} {} != {}\nEntry={}'.format(
                                stat_type,
                                stat_index,
                                key,
                                subkey,
                                expected_stats[key][subkey],
                                subvalue,
                                expected_stats[key]
                            )
                    else:
                        assert expected_stats[key] == value, 'stats.{}[{}].{} {} != {}'.format(
                            stat_type,
                            stat_index,
                            key,
                            expected_stats[key],
                            value
                        )

                stat_index += 1

        return response_data



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
#  Copyright 2013-2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from collections import namedtuple


stat_type = ['timeline', 'desk_transitions', 'featuremedia_updates']

STAT_TYPE = namedtuple('STAT_TYPE', [
    'TIMELINE',
    'DESK_TRANSITIONS',
    'FEATUREMEDIA_UPDATES'
])(*stat_type)


operation = [
    # Create based operations
    'create',
    'fetch',
    'duplicated_from',

    'update',
    'publish',
    'publish_scheduled',
    'deschedule',
    'publish_embargo',
    'rewrite',
    'correct',
    'link',
    'unlink',
    'kill',
    'takedown',

    # Item Movements (between desks etc)
    'spike',
    'unspike',
    'move',
    'move_from',
    'move_to',

    'duplicate',
    'item_lock',
    'item_unlock',

    'mark',
    'unmark',
    'export_highlight',
    'create_highlight',

    'add_featuremedia',
    'change_image_poi',
    'update_featuremedia_poi',
    'remove_featuremedia',
    'update_featuremedia_image'
]

OPERATION = namedtuple('OPERATION', [
    'CREATE',
    'FETCH',
    'DUPLICATED_FROM',

    'UPDATE',
    'PUBLISH',
    'PUBLISH_SCHEDULED',
    'DESCHEDULE',
    'PUBLISH_EMBARGO',
    'REWRITE',
    'CORRECT',
    'LINK',
    'UNLINK',
    'KILL',
    'TAKEDOWN',

    'SPIKE',
    'UNSPIKE',
    'MOVE',
    'MOVE_FROM',
    'MOVE_TO',

    'DUPLICATE',
    'ITEM_LOCK',
    'ITEM_UNLOCK',

    'MARK',
    'UNMARK',
    'EXPORT_HIGHLIGHT',
    'CREATE_HIGHLIGHT',

    'ADD_FEATUREMEDIA',
    'CHANGE_IMAGE_POI',
    'UPDATE_FEATUREMEDIA_POI',
    'REMOVE_FEATUREMEDIA',
    'UPDATE_FEATUREMEDIA_IMAGE'
])(*operation)


ENTER_DESK_OPERATIONS = [
    OPERATION.CREATE,
    OPERATION.FETCH,
    OPERATION.DUPLICATED_FROM,
    OPERATION.MOVE_TO,
    OPERATION.DESCHEDULE,
    OPERATION.UNSPIKE
]

EXIT_DESK_OPERATIONS = [
    OPERATION.PUBLISH,
    OPERATION.SPIKE,
    OPERATION.MOVE_FROM,
    OPERATION.PUBLISH_SCHEDULED,
    OPERATION.PUBLISH_EMBARGO
]


FEATUREMEDIA_OPERATIONS = [
    OPERATION.ADD_FEATUREMEDIA,
    OPERATION.CHANGE_IMAGE_POI,
    OPERATION.UPDATE_FEATUREMEDIA_POI,
    OPERATION.UPDATE_FEATUREMEDIA_IMAGE,
    OPERATION.REMOVE_FEATUREMEDIA
]

OPERATION_NAMES = {
    OPERATION.CREATE: 'Create',
    OPERATION.FETCH: 'Fetch',
    OPERATION.DUPLICATED_FROM: 'Duplicated From',
    OPERATION.UPDATE: 'Save',
    OPERATION.PUBLISH: 'Publish',
    OPERATION.PUBLISH_SCHEDULED: 'Publish Scheduled',
    OPERATION.DESCHEDULE: 'Deschedule',
    OPERATION.PUBLISH_EMBARGO: 'Publish Embargo',
    OPERATION.REWRITE: 'Rewrite',
    OPERATION.CORRECT: 'Correct',
    OPERATION.LINK: 'Link',
    OPERATION.UNLINK: 'Unlink',
    OPERATION.KILL: 'Kill',
    OPERATION.TAKEDOWN: 'Takedown',
    OPERATION.SPIKE: 'Spike',
    OPERATION.UNSPIKE: 'Unspike',
    OPERATION.MOVE: 'Move',
    OPERATION.MOVE_FROM: 'Move From',
    OPERATION.MOVE_TO: 'Move To',
    OPERATION.DUPLICATE: 'Duplicate',
    OPERATION.ITEM_LOCK: 'Lock',
    OPERATION.ITEM_UNLOCK: 'Unlock',
    OPERATION.MARK: 'Mark',
    OPERATION.UNMARK: 'Unmark',
    OPERATION.EXPORT_HIGHLIGHT: 'Export Highlight',
    OPERATION.CREATE_HIGHLIGHT: 'Create Highlight',
    OPERATION.ADD_FEATUREMEDIA: 'Add Featuremedia',
    OPERATION.CHANGE_IMAGE_POI: 'Change POI',
    OPERATION.UPDATE_FEATUREMEDIA_POI: 'Change POI',
    OPERATION.REMOVE_FEATUREMEDIA: 'Remove Featuremedia',
    OPERATION.UPDATE_FEATUREMEDIA_IMAGE: 'Change Image'
}

PUBLISH_OPERATIONS = [
    OPERATION.PUBLISH,
    OPERATION.CORRECT,
    OPERATION.KILL,
    OPERATION.TAKEDOWN,
]



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import Command, command, Option, get_resource_service
from superdesk.logging import logger
from superdesk.errors import SuperdeskApiError
from superdesk.utc import utc_to_local, utcnow, local_to_utc

from flask import current_app as app
from datetime import datetime


class SendScheduledReports(Command):
    """
    Send scheduled reports

    Example:
    ::

        $ python manage.py analytics:send_scheduled_reports
        $ python manage.py analytics:send_scheduled_reports --now

    """

    option_list = [
        Option(
            '--now', '-n',
            dest='now',
            required=False,
            help="Local date/hour in the format '%Y-%m-%dT%H', i.e. 2018-09-13T10"
        )
    ]

    def run(self, now=None):
        if now:
            now_utc = now if isinstance(now, datetime) else local_to_utc(
                app.config['DEFAULT_TIMEZONE'],
                datetime.strptime(now, '%Y-%m-%dT%H')
            )
        else:
            now_utc = utcnow()

        now_local = utc_to_local(app.config['DEFAULT_TIMEZONE'], now_utc)

        logger.info('Starting to send scheduled reports: {}'.format(now_utc))

        schedules = self.get_schedules()

        if len(schedules) < 1:
            logger.info('No enabled schedules found, not continuing')
            return

        # Set now to the beginning of the hour (in local time)
        now_local = now_local.replace(minute=0, second=0, microsecond=0)

        for scheduled_report in schedules:
            schedule_id = str(scheduled_report.get('_id'))

            try:
                if not self.should_send_report(scheduled_report, now_local):
                    logger.info('Scheduled Report {} not scheduled to be sent'.format(schedule_id))
                    continue

                logger.info('Attempting to send Scheduled Report {}'.format(schedule_id))
                self._send_report(scheduled_report)

                # Update the _last_sent of the schedule
                get_resource_service('scheduled_reports').system_update(
                    scheduled_report.get('_id'),
                    {'_last_sent': now_utc},
                    scheduled_report
                )
            except Exception as e:
                logger.error('Failed to generate report for {}. Error: {}'.format(
                    schedule_id,
                    str(e)
                ))
                logger.exception(e)

        logger.info('Completed sending scheduled reports: {}'.format(now_utc))

    @staticmethod
    def get_schedules():
        return list(
            get_resource_service('scheduled_reports').get(
                req=None,
                lookup={'active': {'$eq': True}}
            )
        )

    @staticmethod
    def should_send_report(scheduled_report, now_local):
        # Set now to the beginning of the hour (in local time)
        now_to_hour = now_local.replace(minute=0, second=0, microsecond=0)

        last_sent = None
        if scheduled_report.get('_last_sent'):
            last_sent = utc_to_local(
                app.config['DEFAULT_TIMEZONE'],
                scheduled_report.get('_last_sent')
            ).replace(
                minute=0,
                second=0,
                microsecond=0
            )

        # Fix issue with incorrect schedule attributes being stored
        get_resource_service('scheduled_reports').set_schedule(scheduled_report)
        schedule = scheduled_report.get('schedule') or {}
        schedule_hour = schedule.get('hour', -1)
        schedule_day = schedule.get('day', -1)
        schedule_week_days = schedule.get('week_days') or []

        # Is this report to be run today (Day of the month)?
        # -1 = every day
        if schedule_day > -1 and schedule_day != now_to_hour.day:
            return False

        # Is this report to be run on this week day (i.e. Monday, Wednesday etc)?
        # None or [] = every week day
        week_day = now_to_hour.strftime('%A')
        if len(schedule_week_days) > 0 and week_day not in schedule_week_days:
            return False

        # Is this report to be run on this hour (i.e. 8am)
        # -1 = every hour
        if schedule_hour > -1 and schedule_hour != now_to_hour.hour:
            return False

        # This report has not been run on this hour
        if last_sent is not None and now_to_hour <= last_sent:
            return False

        return True

    @staticmethod
    def _send_report(scheduled_report):
        email_service = get_resource_service('email_report')
        saved_report = get_resource_service('saved_reports').find_one(
            req=None,
            _id=scheduled_report.get('saved_report')
        )

        if not saved_report:
            raise SuperdeskApiError.notFoundError('Saved report not found')

        extra = scheduled_report.get('extra') or {}
        body = extra.get('body') or 'Superdesk Analytics - {}'.format(
            scheduled_report.get('name')
        )

        email_service.post([{
            'report': {
                'type': saved_report.get('report'),
                'params': saved_report.get('params'),
                'mimetype': scheduled_report.get('mimetype'),
                'width': scheduled_report.get('report_width')
            },
            'email': {
                'recipients': scheduled_report.get('recipients'),
                'subject': 'Superdesk Analytics - {}'.format(scheduled_report.get('name')),
                'txt': {'body': body},
                'html': {'body': body}
            }
        }])


command('analytics:send_scheduled_reports', SendScheduledReports())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.services import BaseService
from superdesk.resource import Resource
from superdesk.errors import SuperdeskApiError
from superdesk.lock import lock, unlock
from superdesk.logging import logger
from superdesk.celery_app import celery

from analytics.common import get_report_service, mime_types, MIME_TYPES, get_mime_type_extension
from analytics.reports import generate_report
from .analytics_message import AnalyticsMessage

from flask import current_app as app, render_template
from email.charset import Charset, QP
from base64 import b64decode
from uuid import uuid4
from bson import ObjectId


class EmailReportResource(Resource):
    """Resource to email report charts"""

    endpoint_name = resource_title = url = 'email_report'
    resource_methods = ['POST']
    privileges = {'POST': 'scheduled_reports'}

    schema = {
        'report': {
            'type': 'dict',
            'required': True,
            'schema': {
                'type': {
                    'type': 'string',
                    'required': True
                },
                'params': {
                    'type': 'dict',
                    'required': True
                },
                'mimetype': {
                    'type': 'string',
                    'required': True,
                    'allowed': mime_types,
                    'default': MIME_TYPES.JPEG
                },
                'width': {
                    'type': 'integer',
                    'required': False,
                    'default': 800
                },
                'translations': {
                    'type': 'dict',
                    'required': False
                }
            }
        },
        'email': {
            'type': 'dict',
            'required': True,
            'schema': {
                'sender': {
                    'type': 'string',
                    'required': False
                },
                'recipients': {
                    'type': 'list',
                    'required': True,
                    'schema': {'type': 'string'}
                },
                'subject': {
                    'type': 'string',
                    'required': True
                },
                'txt': {
                    'type': 'dict',
                    'required': True,
                    'schema': {
                        'body': {
                            'type': 'string',
                            'required': False
                        },
                        'template': {
                            'type': 'string',
                            'required': False,
                            'default': 'analytics_scheduled_report.txt'
                        }
                    }
                },
                'html': {
                    'type': 'dict',
                    'required': True,
                    'schema': {
                        'body': {
                            'type': 'string',
                            'required': False
                        },
                        'template': {
                            'type': 'string',
                            'required': False,
                            'default': 'analytics_scheduled_report.html'
                        }
                    }
                }
            }
        },
    }


class EmailReportService(BaseService):
    def create(self, docs, **kwargs):
        for doc in docs:
            attachments = self._gen_attachments(doc.get('report') or {})
            self._email_report(doc.get('email') or {}, attachments)

        # We're not actually saving anything to the database
        # So return empty array here
        return [0]

    @staticmethod
    def _gen_attachments(report):
        report_service = get_report_service(report.get('type'))
        if report_service is None:
            raise SuperdeskApiError.badRequestError(
                'Unknown report type "{}"'.format(report.get('type'))
            )

        if report.get('mimetype') in [
            MIME_TYPES.PNG,
            MIME_TYPES.JPEG,
            MIME_TYPES.GIF,
            MIME_TYPES.PDF,
            MIME_TYPES.SVG
        ]:
            # This mimetype is handled by highcharts, so generate the highcharts config
            return_type = 'highcharts_config'
        elif report.get('mimetype') == MIME_TYPES.CSV:
            return_type = MIME_TYPES.CSV
        else:
            return_type = 'aggregations'

        generated_report = list(report_service.get(
            req=None,
            params=report.get('params') or {},
            translations=report.get('translations') or {},
            return_type=return_type
        ))[0]

        if return_type == 'highcharts_config':
            options = generated_report.get('highcharts')
        elif return_type == MIME_TYPES.CSV:
            options = [generated_report.get('csv')]
        else:
            options = []

        attachments = []

        i = 1
        for option in options:
            mime_type = report.get('mimetype')
            report_width = report.get('width') or 800

            if isinstance(option, dict) and option.get('type') == 'table':
                mime_type = MIME_TYPES.HTML

            try:
                attachments.append({
                    'file': generate_report(
                        option,
                        mimetype=mime_type,
                        base64=True,
                        width=report_width
                    ),
                    'mimetype': mime_type,
                    'filename': 'chart_{}.{}'.format(i, get_mime_type_extension(mime_type)),
                    'width': report_width
                })
                i += 1
            except Exception as e:
                logger.error('Failed to generate chart.')
                logger.exception(e)

        return attachments

    @staticmethod
    def _email_report(email, attachments):
        txt = email.get('txt') or {}
        html = email.get('html') or {}

        send_email_report.apply_async(
            kwargs={
                '_id': str(ObjectId()),
                'subject': email.get('subject'),
                'sender': email.get('sender') or app.config['ADMINS'][0],
                'recipients': email.get('recipients'),
                'text_body': txt.get('body') or '',
                'html_body': html.get('body') or '',
                'attachments': attachments,
                'txt_template': txt.get('template'),
                'html_template': html.get('template')
            }
        )


@celery.task(bind=True, max_retries=3, soft_time_limit=120)
def send_email_report(
    self,
    _id,
    subject,
    sender,
    recipients,
    text_body='',
    html_body='',
    cc=None,
    bcc=None,
    attachments=None,
    txt_template='analytics_scheduled_report.txt',
    html_template='analytics_scheduled_report.html'
):
    lock_id = 'analytics_email:{}'.format(str(_id))
    if not lock(lock_id, expire=120):
        return

    try:
        charset = Charset('utf-8')
        charset.header_encoding = QP
        charset.body_encoding = QP

        msg = AnalyticsMessage(
            subject,
            sender=sender,
            recipients=recipients,
            cc=cc,
            bcc=bcc,
            body=text_body,
            charset=charset
        )

        reports = []

        if attachments is not None:
            for attachment in attachments:
                try:
                    uuid = str(uuid4())
                    if attachment.get('mimetype') == MIME_TYPES.HTML:
                        reports.append({
                            'id': uuid,
                            'type': 'html',
                            'html': attachment.get('file')
                        })
                    else:
                        msg.attach(
                            filename=attachment.get('filename'),
                            content_type='{}; name="{}"'.format(attachment.get('mimetype'),
                                                                attachment.get('filename')),
                            data=b64decode(attachment.get('file')),
                            disposition='attachment',
                            headers={
                                'Content-ID': '<{}>'.format(uuid),
                                'X-Attachment-Id': uuid
                            }.items()
                        )

                        reports.append({
                            'id': uuid,
                            'type': 'image',
                            'filename': attachment.get('filename'),
                            'width': attachment.get('width')
                        })

                        msg.body += '\n[image: {}]'.format(attachment.get('filename'))
                except Exception as e:
                    logger.error('Failed to generate attachment.')
                    logger.exception(e)

        msg.body = render_template(
            txt_template,
            text_body=text_body,
            reports=reports
        )

        msg.html = render_template(
            html_template,
            html_body=html_body.replace('\r', '').replace('\n', '<br>'),
            reports=reports
        )

        return app.mail.send(msg)
    except Exception as e:
        logger.error('Failed to send report email. Error: {}'.format(str(e)))
        logger.exception(e)
    finally:
        unlock(lock_id, remove=True)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
#  Copyright 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from copy import deepcopy

from superdesk.resource import Resource
from superdesk import Service, get_resource_service
from superdesk.utils import ListCursor

from analytics.common import registered_reports, DATE_FILTERS, CHART_TYPES, REPORT_CONFIG


class ReportConfigsResource(Resource):
    endpoint_name = resource_title = 'report_configs'
    schema = {
        '_id': {
            'type': 'string',
            'required': True,
            'minlength': 1
        },
        'name': {'type': 'string'},
        'enabled': {
            'type': 'boolean',
            'default': True
        },
        REPORT_CONFIG.DATE_FILTERS: {'type': 'dict'},
        REPORT_CONFIG.CHART_TYPES: {'type': 'dict'},
        REPORT_CONFIG.DEFAULT_PARAMS: {'type': 'dict'}
    }


base_config = {
    REPORT_CONFIG.CHART_TYPES: {
        CHART_TYPES.BAR: {'enabled': True},
        CHART_TYPES.COLUMN: {'enabled': True},
        CHART_TYPES.TABLE: {'enabled': True},
        CHART_TYPES.AREA: {'enabled': False},
        CHART_TYPES.LINE: {'enabled': False},
        CHART_TYPES.PIE: {'enabled': False},
        CHART_TYPES.SCATTER: {'enabled': False},
        CHART_TYPES.SPLINE: {'enabled': False}
    },
    REPORT_CONFIG.DEFAULT_PARAMS: {},
    REPORT_CONFIG.DATE_FILTERS: {
        # ABSOLUTE
        DATE_FILTERS.RANGE: {'enabled': True},
        DATE_FILTERS.DAY: {'enabled': True},

        # HOURS
        DATE_FILTERS.RELATIVE_HOURS: {
            'enabled': True,
            'max': 72
        },

        # DAYS
        DATE_FILTERS.RELATIVE_DAYS: {
            'enabled': True,
            'max': 31
        },
        DATE_FILTERS.YESTERDAY: {'enabled': True},
        DATE_FILTERS.TODAY: {'enabled': True},

        # WEEKS
        DATE_FILTERS.RELATIVE_WEEKS: {
            'enabled': True,
            'max': 52
        },
        DATE_FILTERS.LAST_WEEK: {'enabled': True},
        DATE_FILTERS.THIS_WEEK: {'enabled': True},

        # MONTHS
        DATE_FILTERS.RELATIVE_MONTHS: {
            'enabled': True,
            'max': 12
        },
        DATE_FILTERS.LAST_MONTH: {'enabled': True},
        DATE_FILTERS.THIS_MONTH: {'enabled': True},

        # YEARS
        DATE_FILTERS.LAST_YEAR: {'enabled': True},
        DATE_FILTERS.THIS_YEAR: {'enabled': True}
    }
}


class ReportConfigsService(Service):
    def get(self, req, lookup):
        configs = list(super().get(req, lookup))
        merged_configs = []

        for report_id, endpoint in registered_reports.items():
            service = get_resource_service(endpoint)
            default_config = deepcopy(getattr(service, 'defaultConfig', {}))

            for key, val in base_config.items():
                if key not in default_config:
                    default_config[key] = val

            config = next((
                c for c in configs
                if c.get('_id') == report_id
            ), None)

            if config is None:
                default_config['_id'] = report_id
                merged_configs.append(default_config)
            else:
                self.merge_config(config, default_config)
                merged_configs.append(config)

        return ListCursor(merged_configs)

    def merge_config(self, config, default_config):
        """Merge the default config and config from mongo

        Attributes that are not in the default values will be omitted
        This is how you define if a particular filter/chart type is supported
        """
        updated_config = deepcopy(default_config)
        updated_config.setdefault('date_filters', {})
        updated_config.setdefault('chart_types', {})

        new_date_filters = config.get('date_filters') or {}
        new_chart_types = config.get('chart_types') or {}

        for key, value in default_config['date_filters'].items():
            updated_config['date_filters'][key] = new_date_filters.get(key) or value

        for key, value in default_config['chart_types'].items():
            updated_config['chart_types'][key] = new_chart_types.get(key) or value

        config['date_filters'] = updated_config['date_filters']
        config['chart_types'] = updated_config['chart_types']
        config['default_params'] = config.get('default_params') or default_config['default_params']



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from superdesk import get_resource_service

from analytics.common import get_cv_by_qcode, DATE_FILTERS
from analytics.chart_config import SDChart
from analytics.stats.common import OPERATION_NAMES

from datetime import datetime, timedelta


class ChartConfig:
    """Class to generate Highcharts config"""

    defaultConfig = {
        'credits': {'enabled': False}
    }

    def __init__(self, chart_id, chart_type):
        """Initialise the data for the chart config

        :param str chart_id: The id to be given to the chart
        :param str chart_type: The qcode of the chart type to generate
        """

        self.id = chart_id
        self.config = {}
        self.title = ''
        self.subtitle = ''
        self.chart_type = chart_type
        self.sources = []
        self.sort_order = 'desc'

        self.translations = {}

    def is_multi_source(self):
        """Returns True if this chart has multiple data sources

        :return bool: True if chart has more than 1 data source
        """

        return len(self.sources) > 1

    def _get_source(self, index):
        """Returns the source's field and data attributes

        :param int index: The index of self.sources to use
        :return dict: Source's field and data attributes
        """

        try:
            source = self.sources[index]
        except IndexError:
            source = {}

        return {
            'field': source.get('field') or '',
            'data': source.get('data') or {}
        }

    def get_parent(self):
        """Returns the parent data source's field and data attributes

        :return dict: Parent field and data attributes
        """

        return self._get_source(0)

    def get_child(self):
        """Returns the child data source's field and data attributes

        :return dict: Child field and data attributes
        """

        return self._get_source(1)

    def get_title(self):
        """Generates the title string to use for the chart

        :return str: Title for the chart
        """

        return self.title

    def get_subtitle(self):
        """Generates the subtitle string to use for the chart

        :return str: Subtitle for the chart
        """

        return self.subtitle

    def get_source_name(self, field):
        """Generates the name for the given source

        :param str field: The field attribute of the source, i.e. anpa_category.qcode
        :return str: Data source name
        """
        return self._get_translation_title(field)

    def get_y_axis_title(self):
        """Generates the title for the Y Axis (defaults to 'Published Stories')

        :return str: The title for the Y Axis
        """

        return 'Published Stories'

    def get_source_title(self, field, qcode):
        """Generates the name for the specific data source

        :param str field: The field attribute of the source (i.e. anpa_category.qcode)
        :param str qcode: The key value for the specific data source
        :return str: Name of the source type
        """
        return self._get_translation_names(field).get(qcode) or qcode

    def get_sorted_keys(self, data):
        """Generates array of keys based on sorting of the data (using this.sortOrder)

        :param dict data: The source data to get the keys for
        :return list: Key names
        """

        return self.get_single_sorted_keys(data) if not self.is_multi_source() \
            else self.get_multi_sorted_keys(data)

    def get_single_sorted_keys(self, data):
        """Generates array of keys for single series data

        :param dict data: The source data to get the keys for
        :return list: Key names
        """

        return [
            category for category, count
            in sorted(
                data.items(),
                key=lambda kv: kv[1] if self.sort_order == 'asc' else -kv[1]
            )
        ]

    def get_multi_sorted_keys(self, data):
        """Generates array of keys for stacked series data

        :param dict data:
        :return list: Key names
        """

        return [
            category for category, count
            in sorted(
                data.items(),
                key=lambda kv: sum(kv[1].values()) if self.sort_order == 'asc'
                else -sum(kv[1].values())
            )
        ]

    def add_source(self, field, data):
        """Adds the provided sources field and data to this chart config

        :param str field: The sources field attribute (i.e. anpa_category.qcode)
        :param dict data: A dictionary containing the source data
        """

        self.sources.append({
            'field': field,
            'data': data
        })

    def gen_highcharts_config(self):
        """Generates and returns the Highcharts config

        :return dict: Highcharts config
        """
        config_type = 'table' if self.chart_type == 'table' else 'highcharts'
        chart = SDChart.Chart(
            self.id,
            chart_type=config_type,
            title=self.get_title(),
            subtitle=self.get_subtitle(),
            default_config=self.defaultConfig
        )

        chart.translations = self.translations
        chart.tooltip_point = ''

        parent = self.get_parent()

        axis_options = {
            'type': 'category',
            'default_chart_type': 'column' if self.chart_type == 'table' else self.chart_type,
            'y_title': self.get_y_axis_title(),
            'x_title': chart.get_translation_title(parent['field']),
            'category_field': parent['field'],
            'categories': self.get_sorted_keys(parent['data']),
        }

        if not self.is_multi_source():
            chart.tooltip_header = '{point.x}: {point.y}'
            chart.data_labels = True
            chart.colour_by_point = True
            axis_options['stack_labels'] = False

            axis = chart.add_axis() \
                .set_options(**axis_options)

            axis.add_series().set_options(
                field=parent['field'],
                data=[
                    parent['data'][key]
                    for key in self.get_sorted_keys(parent['data'])
                ]
            )
        else:
            child = self.get_child()
            chart.legend_title = self.get_source_name(child['field'])
            chart.tooltip_header = '{series.name}/{point.x}: {point.y}'
            chart.data_labels = False
            chart.colour_by_point = False
            axis_options['stack_labels'] = True

            axis = chart.add_axis() \
                .set_options(**axis_options)

            for child_key in child['data'].keys():
                axis.add_series().set_options(
                    field=child['field'],
                    name=child_key,
                    stack=0,
                    stackType='normal',
                    data=[
                        counts.get(child_key) or 0
                        for counts in [
                            parent['data'][key] for key in self.get_sorted_keys(parent['data'])
                        ]
                    ]
                )

        return chart.gen_config()

    def gen_config(self):
        """High level function to generates the Highcharts/Table config based on chart options

        :return dict: Highchart or Table config
        """
        self.load_translations()
        self.config = self.gen_highcharts_config()

        return self.config

    @staticmethod
    def get_translations(fields):
        chart = ChartConfig('tmp', 'chart')
        for field in fields:
            chart.load_translations_for_field(field)
        return chart.translations

    def load_translations(self, parent_field=None, child_field=None):
        """Loads data for translating id/qcode to display names

        :param str parent_field: Name of the first field (defaults to Parent)
        :param str child_field: Name of the second field (defaults to Child)
        """
        if parent_field is None:
            parent = self.get_parent()
            parent_field = parent['field']

        if child_field is None:
            child = self.get_child()
            child_field = child['field']

        self.load_translations_for_field(parent_field)
        self.load_translations_for_field(child_field)

    def load_translations_for_field(self, field):
        # If a translation for this field has already been loaded
        # then don't bother re-loading the translation for it
        if field in self.translations:
            return

        elif field == 'task.desk':
            self._set_translation(
                'task.desk',
                'Desk',
                {
                    str(desk.get('_id')): desk.get('name')
                    for desk in list(get_resource_service('desks').get(req=None, lookup={}))
                }
            )
        elif field == 'task.user':
            self._set_translation(
                'task.user',
                'User',
                {
                    str(user.get('_id')): user.get('display_name')
                    for user in list(get_resource_service('users').get(req=None, lookup={}))
                }
            )
        elif field == 'anpa_category.qcode':
            self._set_translation(
                'anpa_category.qcode',
                'Category',
                get_cv_by_qcode('categories', 'name')
            )
        elif field == 'genre.qcode':
            self._set_translation(
                'genre.qcode',
                'Genre',
                get_cv_by_qcode('genre', 'name')
            )
        elif field == 'subject.qcode':
            self._set_translation(
                'subject.qcode',
                'Subject',
                get_cv_by_qcode('subjectcodes', 'name')
            )
        elif field == 'urgency':
            self._set_translation(
                'urgency',
                'Urgency',
                get_cv_by_qcode('urgency', 'name')
            )
        elif field == 'state':
            self._set_translation(
                'state',
                'State',
                {
                    'published': 'Published',
                    'killed': 'Killed',
                    'corrected': 'Corrected',
                    'updated': 'Updated'
                }
            )
        elif field == 'source':
            self._set_translation('source', 'Source')
        elif field == 'operation':
            self._set_translation('operation', 'Operation', OPERATION_NAMES)
        elif field == 'authors.parent':
            self._set_translation(
                'authors.parent',
                'Author',
                {
                    str(user.get('_id')): user.get('display_name')
                    for user in list(get_resource_service('users').get(req=None, lookup={}))
                }
            )

    def _set_translation(self, field, title, names=None):
        """Saves the provided field translations

        :param str field: The name of the field for this translation
        :param str title: The title of the field name
        :param dict names: Map of id/qcode to display names
        """
        self.translations[field.replace('.', '_')] = {
            'title': title,
            'names': names or {}
        }

    def _get_translations(self, field):
        """Helper function to get the translations for a field

        :param str field: Name of the field to get translations for
        :return dict: Title and name translations
        """
        return self.translations.get(field.replace('.', '_')) or {}

    def _get_translation_title(self, field):
        """Helper function to get the translated title for a field

        :param str field: Name of the field to get translated title for
        :return str: Title of the field
        """
        return self._get_translations(field).get('title') or field

    def _get_translation_names(self, field):
        """Helper function to get the translated id/qcode to name map for a field

        :param str field: Name of the field to get translated names for
        :return dict: id/qcode to name map
        """
        return self._get_translations(field).get('names') or {}

    @staticmethod
    def gen_subtitle_for_dates(params):
        chart = params.get('chart') or {}

        if chart.get('subtitle'):
            return chart['subtitle']

        dates = params.get('dates') or {}
        date_filter = dates.get('filter')

        # ABSOLUTE
        if date_filter == DATE_FILTERS.RANGE:
            start = datetime.strptime(dates.get('start'), '%Y-%m-%d')
            end = datetime.strptime(dates.get('end'), '%Y-%m-%d')

            return '{} - {}'.format(
                start.strftime('%B %-d, %Y'),
                end.strftime('%B %-d, %Y')
            )
        elif date_filter == DATE_FILTERS.DAY:
            date_str = params.get('date') or dates.get('date')
            date = datetime.strptime(date_str, '%Y-%m-%d')

            return date.strftime('%A %-d %B %Y')

        # HOURS
        elif date_filter == DATE_FILTERS.RELATIVE_HOURS:
            hours = dates.get('relative')

            return 'Last {} hours'.format(hours)

        # DAYS
        elif date_filter == DATE_FILTERS.RELATIVE_DAYS:
            days = dates.get('relative')

            return 'Last {} days'.format(days)
        elif date_filter == DATE_FILTERS.YESTERDAY:
            return (datetime.today() - timedelta(days=1)) \
                .strftime('%A %-d %B %Y')
        elif date_filter == DATE_FILTERS.TODAY:
            return datetime.today().strftime('%A %-d %B %Y')

        # WEEKS
        elif date_filter == DATE_FILTERS.RELATIVE_WEEKS:
            weeks = dates.get('relative')

            return 'Last {} weeks'.format(weeks)
        elif date_filter == DATE_FILTERS.LAST_WEEK:
            week = datetime.today() - timedelta(weeks=1)
            start = week - timedelta(days=week.weekday() + 1)
            end = start + timedelta(days=6)

            return '{} - {}'.format(
                start.strftime('%B %-d, %Y'),
                end.strftime('%B %-d, %Y')
            )
        elif date_filter == DATE_FILTERS.THIS_WEEK:
            week = datetime.today()
            start = week - timedelta(days=week.weekday() + 1)
            end = start + timedelta(days=6)

            return '{} - {}'.format(
                start.strftime('%B %-d, %Y'),
                end.strftime('%B %-d, %Y')
            )

        # MONTHS
        elif date_filter == DATE_FILTERS.RELATIVE_MONTHS:
            months = dates.get('relative')

            return 'Last {} months'.format(months)
        elif date_filter == DATE_FILTERS.LAST_MONTH:
            first = datetime.today().replace(day=1)
            month = first - timedelta(days=1)

            return month.strftime('%B %Y')
        elif date_filter == DATE_FILTERS.THIS_MONTH:
            month = datetime.today()

            return month.strftime('%B %Y')

        # YEARS
        elif date_filter == DATE_FILTERS.LAST_YEAR:
            year = datetime.today() - timedelta(weeks=52)

            return year.strftime('%Y')
        elif date_filter == DATE_FILTERS.THIS_YEAR:
            year = datetime.today()

            return year.strftime('%Y')

        return None



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource

from analytics.base_report import BaseReportService
from analytics.chart_config import ChartConfig
from superdesk import get_resource_service


class PublishingPerformanceReportResource(Resource):
    """Publishing Performance Report schema
    """

    item_methods = ['GET']
    resource_methods = ['GET']
    privileges = {'GET': 'publishing_performance_report'}


class PublishingPerformanceReportService(BaseReportService):
    repos = ['published', 'archived']
    aggregations = {
        'source': {
            'terms': {
                'field': 'source',
                'size': 0,
            }
        }
    }

    @staticmethod
    def _get_aggregation_query(agg):
        include = agg.get('filter') or 'all'

        query = {
            'terms': {
                'field': agg.get('field'),
                'size': agg.get('size') or 0
            },
            'aggs': {
                'no_rewrite_of': {
                    'filter': {
                        'not': {
                            'filter': {
                                'exists': {
                                    'field': 'rewrite_of'
                                }
                            }
                        }
                    },
                    'aggs': {
                        'state': {
                            'terms': {
                                'field': 'state'
                            }
                        }
                    }
                },
                'rewrite_of': {
                    'filter': {
                        'exists': {
                            'field': 'rewrite_of'
                        }
                    },
                    'aggs': {
                        'state': {
                            'terms': {
                                'field': 'state'
                            }
                        }
                    }
                }
            }
        }

        if include != 'all':
            query['terms']['include'] = include
            query['terms']['min_doc_count'] = 0

        return query

    def get_aggregations(self, params, args):
        aggs = params.get('aggs') or {}

        if not aggs or not (aggs.get('group') or {}).get('field'):
            return PublishingPerformanceReportService.aggregations

        aggregations = {
            'parent': self._get_aggregation_query(aggs['group'])
        }

        return aggregations

    def generate_report(self, docs, args):
        """Returns the publishing statistics

        :param docs: document used for generating the statistics
        :return dict: report
        """
        agg_buckets = self.get_aggregation_buckets(getattr(docs, 'hits'), ['parent'])

        states = ['killed', 'corrected', 'updated', 'published', 'recalled']

        report = {
            'groups': {},
            'subgroups': {state: 0 for state in states}
        }

        for parent in agg_buckets.get('parent') or []:
            parent_key = parent.get('key')

            if not parent_key:
                continue

            report['groups'][parent_key] = {state: 0 for state in states}

            no_rewrite_of = (parent.get('no_rewrite_of') or {}).get('state') or {}
            rewrite_of = (parent.get('rewrite_of') or {}).get('state') or {}

            for child in no_rewrite_of.get('buckets') or []:
                state_key = child.get('key')

                if not state_key or state_key not in states:
                    continue

                doc_count = child.get('doc_count') or 0

                report['groups'][parent_key][state_key] += doc_count
                report['subgroups'][state_key] += doc_count

            for child in rewrite_of.get('buckets') or []:
                state_key = child.get('key')

                if not state_key or state_key not in states:
                    continue

                if state_key == 'published':
                    state_key = 'updated'

                doc_count = child.get('doc_count') or 0

                report['groups'][parent_key][state_key] += doc_count
                report['subgroups'][state_key] += doc_count

        if args and args.get('show_all_desks'):

            desk_with_no_articles = {'killed': 0, 'corrected': 0, 'updated': 0, 'published': 0, 'recalled': 0}
            report_groups = list(report['groups'])
            desks = get_resource_service('desks').get(req=None, lookup={})

            for desk in list(desks):
                if str(desk['_id']) not in report_groups:
                    report['groups'][str(desk['_id'])] = desk_with_no_articles

        return report

    def generate_highcharts_config(self, docs, args):
        params = args.get('params') or {}
        aggs = args.get('aggs') or {}
        group = aggs.get('group') or {}
        subgroup = aggs.get('subgroup') or {}
        translations = args.get('translations') or {}

        chart = params.get('chart') or {}
        chart_type = chart.get('type') or 'bar'

        report = self.generate_report(docs, args)

        chart_config = ChartConfig('content_publishing', chart_type)

        group_keys = list(report['groups'].keys())
        if len(group_keys) == 1 and report.get('subgroups'):
            chart_config.add_source(
                subgroup.get('field'),
                report['subgroups']
            )
            chart_config.load_translations(group.get('field'))
        else:
            chart_config.add_source(group.get('field'), report.get('groups'))
            if report.get('subgroups'):
                chart_config.add_source(subgroup.get('field'), report['subgroups'])

        def gen_title():
            if chart.get('title'):
                return chart['title']

            group_type = group.get('field')
            group_title = chart_config.get_source_name(group_type)

            if len(group_keys) == 1 and report.get('subgroups'):
                data_name = chart_config.get_source_title(
                    group_type,
                    group_keys[0]
                )

                return 'Published Stories for {}: {}'.format(
                    group_title,
                    data_name
                )

            return 'Published Stories per {}'.format(group_title)

        def gen_subtitle():
            return ChartConfig.gen_subtitle_for_dates(params)

        chart_config.get_title = gen_title
        chart_config.get_subtitle = gen_subtitle
        chart_config.sort_order = chart.get('sort_order') or 'desc'

        chart_config.translations = translations

        report['highcharts'] = [chart_config.gen_config()]

        return report



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import get_resource_service
from superdesk.services import BaseService
from superdesk.resource import Resource
from superdesk.notification import push_notification
from superdesk.errors import SuperdeskApiError

from analytics.common import mime_types, MIME_TYPES

from collections import namedtuple
from copy import deepcopy

frequencies = ['hourly', 'daily', 'weekly', 'monthly']
FREQUENCIES = namedtuple('FREQUENCIES', ['HOURLY', 'DAILY', 'WEEKLY', 'MONTHLY'])(*frequencies)


class ScheduledReportsResource(Resource):
    endpoint_name = resource_title = 'scheduled_reports'
    url = 'scheduled_reports'
    item_methods = ['GET', 'PATCH', 'DELETE']
    resource_methods = ['GET', 'POST']
    privileges = {
        'GET': 'scheduled_reports',
        'POST': 'scheduled_reports',
        'PATCH': 'scheduled_reports',
        'DELETE': 'scheduled_reports',
    }

    schema = {
        'saved_report': Resource.rel('saved_reports'),
        'report_type': {
            'type': 'string',
            'required': True
        },
        'schedule': {
            'type': 'dict',
            'required': True,
            'schema': {
                'frequency': {
                    'type': 'string',
                    'required': True,
                    'allowed': [
                        'hourly',
                        'daily',
                        'weekly',
                        'monthly'
                    ]
                },
                'hour': {
                    'type': 'integer',
                    'default': -1
                },
                'day': {
                    'type': 'integer',
                    'default': -1
                },
                'week_days': {
                    'type': 'list',
                    'schema': {
                        'type': 'string',
                        'allowed': [
                            'Sunday',
                            'Monday',
                            'Tuesday',
                            'Wednesday',
                            'Thursday',
                            'Friday',
                            'Saturday'
                        ]
                    }
                }
            }
        },
        'mimetype': {
            'type': 'string',
            'required': True,
            'allowed': mime_types,
            'default': MIME_TYPES.PNG
        },
        'report_width': {
            'type': 'integer',
            'default': 800
        },
        'transmitter': {
            'type': 'string',
            'required': True,
            'allowed': ['email']
        },
        'recipients': {
            'type': 'list',
            'required': True,
            'schema': {'type': 'string'}
        },
        'name': {  # used to construct email subject
            'type': 'string',
            'required': True
        },
        'description': {
            'type': 'string'
        },
        'active': {
            'type': 'boolean',
            'default': False
        },
        'extra': {
            'type': 'dict'  # i.e. email body
        },
        '_last_sent': {
            'type': 'datetime'
        }
    }


class ScheduledReportsService(BaseService):
    def on_created(self, docs):
        for doc in docs:
            self.set_schedule(doc)
            self._validate_on_create_or_update(doc)
            self._push_notification(doc, 'create')

    def on_updated(self, updates, original):
        self.set_schedule(updates)
        doc = deepcopy(original)
        doc.update(updates)
        self._validate_on_create_or_update(doc)
        self._push_notification(original, 'update')

    def on_deleted(self, doc):
        self._push_notification(doc, 'delete')

    def set_schedule(self, updates):
        # Sometimes 'schedule' is not in the updates provided
        # Eve/Cerberus will ensure the document has 'schedule' set
        # as it is configured as required
        if not updates.get('schedule'):
            return

        schedule = updates.get('schedule') or {}
        hour = schedule.get('hour', -1)
        day = schedule.get('day', -1)
        week_days = schedule.get('week_days') or []
        frequency = schedule.get('frequency') or 'hourly'

        # Fix issue with incorrect schedule attributes being stored
        if frequency == 'hourly':
            updates['schedule'].update({
                'frequency': 'hourly',
                'hour': -1,
                'day': -1,
                'week_days': []
            })
        elif frequency == 'daily':
            updates['schedule'].update({
                'frequency': 'daily',
                'hour': hour,
                'day': -1,
                'week_days': []
            })
        elif frequency == 'weekly':
            updates['schedule'].update({
                'frequency': 'weekly',
                'hour': hour,
                'day': -1,
                'week_days': week_days
            })
        elif frequency == 'monthly':
            updates['schedule'].update({
                'frequency': 'monthly',
                'hour': hour,
                'day': day,
                'week_days': []
            })

    @staticmethod
    def _validate_on_create_or_update(doc):
        saved_service = get_resource_service('saved_reports')
        saved_report = saved_service.find_one(
            req=None,
            _id=doc['saved_report']
        )

        if not saved_report.get('is_global'):
            raise SuperdeskApiError.badRequestError(
                'A schedule must be attached to a global saved report'
            )

    @staticmethod
    def _push_notification(doc, operation):
        push_notification(
            'scheduled_reports:update',
            operation=operation,
            schedule_id=str(doc.get('_id'))
        )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.tests import TestCase as _TestCase, update_config, setup
from superdesk.factory.app import get_app


class TestCase(_TestCase):

    def setUp(self):
        config = {
            'INSTALLED_APPS': ['analytics'],
            'STATISTICS_MONGO_DBNAME': 'sptests_statistics'
        }
        update_config(config)
        self.app = get_app(config)
        setup.app = self.app
        super().setUp()



from setuptools import setup, find_packages
from setuptools.command.develop import develop
from setuptools.command.install import install
from subprocess import check_call
from os import path
from distutils import log


def install_highcharts():
    try:
        check_call(
            "./install-highcharts-export-server.sh",
            cwd=path.realpath('server/scripts')
        )
    except Exception as e:
        log.error('\t**NodeJs not found, report scheduling will not work**:\n\t{}'.format(e))


class PostDevelopCommand(develop):
    """Post-installation for development mode."""
    def run(self):
        self.execute(install_highcharts, ())
        develop.run(self)


class PostInstallCommand(install):
    """Post-installation for installation mode."""
    def run(self):
        self.execute(install_highcharts, ())
        install.run(self)


setup(
    name="superdesk-analytics",
    version="1.33.3",
    package_dir={'': 'server'},
    packages=find_packages('server'),
    author='Sourcefabric',
    author_email='contact@sourcefabric.org',
    license='MIT',
    url='https://github.com/superdesk/superdesk-analytics',
    cmdclass={
        'develop': PostDevelopCommand,
        'install': PostInstallCommand
    }
)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
#  Copyright 2013-2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from superdesk import get_resource_service, json
from superdesk.services import BaseService
from superdesk.resource import Resource, not_indexed, not_analyzed, not_enabled
from superdesk.metadata.item import metadata_schema, FORMAT, ITEM_TYPE, ITEM_STATE, \
    SCHEDULE_SETTINGS, BYLINE
from superdesk.metadata.utils import item_url
from apps.archive.common import ARCHIVE_SCHEMA_FIELDS

from analytics.stats.common import STAT_TYPE

from eve.utils import config, ParsedRequest, date_to_str


class ArchiveStatisticsResource(Resource):
    endpoint_name = resource_title = url = 'archive_statistics'
    item_url = item_url
    item_methods = ['GET', 'PATCH', 'DELETE']
    resource_methods = ['GET', 'POST']
    datasource = {
        'source': 'archive_statistics',
        'search_backend': 'elastic',
    }

    mongo_prefix = 'STATISTICS_MONGO'
    elastic_prefix = 'STATISTICS_ELASTIC'

    query_objectid_as_string = True

    schema = {
        config.ID_FIELD: metadata_schema[config.ID_FIELD],
        'guid': metadata_schema['guid'],
        'stats_type': {'type': 'string'},
        'stats': {
            'type': 'dict',
            'schema': {
                STAT_TYPE.TIMELINE: {
                    'type': 'list',
                    'schema': {
                        'type': 'dict',
                        'schema': {
                            'history_id': {
                                'type': 'string',
                                'mapping': not_indexed
                            },
                            'related_history_id': {
                                'type': 'string',
                                'mapping': not_indexed
                            },
                            'operation': {'type': 'string'},
                            'operation_created': {'type': 'datetime'},
                            ITEM_STATE: metadata_schema[ITEM_STATE],
                            'pubstatus': {'type': 'string'},
                            'word_count': metadata_schema['word_count'],
                            'par_count': metadata_schema['word_count'],
                            'task': {
                                'type': 'dict',
                                'schema': {
                                    'user': Resource.rel('users'),
                                    'desk': Resource.rel('desks', nullable=True),
                                    'stage': Resource.rel('stages', nullable=True)
                                }
                            },
                            'update': {
                                'type': 'dict',
                                'mapping': not_enabled
                            }
                        }
                    },
                    'mapping': {
                        'type': 'nested',
                        'properties': {
                            'history_id': not_indexed,
                            'related_history_id': not_indexed,
                            'operation': {'type': 'string'},
                            'operation_created': {'type': 'date'},
                            ITEM_STATE: {'type': 'string'},
                            'pubstatus': {'type': 'string'},
                            'word_count': {'type': 'integer'},
                            'par_count': {'type': 'integer'},
                            'task': {
                                'type': 'object',
                                'properties': {
                                    'user': not_analyzed,
                                    'desk': not_analyzed,
                                    'stage': not_analyzed
                                }
                            },
                            'update': not_enabled
                        }
                    }
                },
                STAT_TYPE.DESK_TRANSITIONS: {
                    'type': 'list',
                    'schema': {
                        'type': 'dict',
                        'schema': {
                            'user': Resource.rel('users'),
                            'desk': Resource.rel('desks', nullable=True),
                            'stage': Resource.rel('stages', nullable=True),
                            'entered': {'type': 'datetime'},
                            'entered_operation': {'type': 'string'},
                            'exited': {'type': 'datetime'},
                            'exited_operation': {'type': 'string'},
                            'duration': {'type': 'integer'},
                        }
                    },
                    'mapping': {
                        'type': 'nested',
                        'properties': {
                            'user': not_analyzed,
                            'desk': not_analyzed,
                            'stage': not_analyzed,
                            'entered': {'type': 'date'},
                            'entered_operation': {'type': 'string'},
                            'exited': {'type': 'date'},
                            'exited_operation': {'type': 'string'},
                            'duration': {'type': 'integer'}
                        }
                    }
                },
                STAT_TYPE.FEATUREMEDIA_UPDATES: {
                    'type': 'list',
                    'schema': {
                        'type': 'dict',
                        'schema': {
                            'related_history_id': {
                                'type': 'string',
                                'mapping': not_analyzed
                            },
                            'operation': {'type': 'string'},
                            'operation_created': {'type': 'datetime'},
                            ITEM_STATE: metadata_schema[ITEM_STATE],
                            'pubstatus': {'type': 'string'},
                            'word_count': metadata_schema['word_count'],
                            'par_count': metadata_schema['word_count'],
                            'task': {
                                'type': 'dict',
                                'schema': {
                                    'user': Resource.rel('users'),
                                    'desk': Resource.rel('desks', nullable=True),
                                    'stage': Resource.rel('stages', nullable=True)
                                }
                            },
                            'renditions': {
                                'type': 'dict',
                                'mapping': not_indexed
                            },
                            'poi': {
                                'type': 'dict',
                                'mapping': not_enabled
                            }
                        }
                    },
                    'mapping': {
                        'type': 'nested',
                        'properties': {
                            'related_history_id': not_analyzed,
                            'operation': {'type': 'string'},
                            'operation_created': {'type': 'date'},
                            ITEM_STATE: {'type': 'string'},
                            'pubstatus': {'type': 'string'},
                            'word_count': {'type': 'integer'},
                            'par_count': {'type': 'integer'},
                            'task': {
                                'type': 'object',
                                'properties': {
                                    'user': not_analyzed,
                                    'desk': not_analyzed,
                                    'stage': not_analyzed
                                }
                            },
                            'renditions': not_enabled,
                            'poi': not_enabled
                        }
                    }
                },
            }
        },

        # Metadata
        'original_creator': metadata_schema['original_creator'],
        'version': metadata_schema['version'],
        'version_creator': metadata_schema['version_creator'],
        'versioncreated': metadata_schema['versioncreated'],
        'firstpublished': metadata_schema['firstpublished'],
        'firstcreated': metadata_schema['firstcreated'],
        'source': metadata_schema['source'],
        'original_source': metadata_schema['original_source'],
        'ingest_provider': metadata_schema['ingest_provider'],
        'anpa_category': metadata_schema['anpa_category'],
        'subject': metadata_schema['subject'],
        'genre': metadata_schema['genre'],
        'company_codes': metadata_schema['company_codes'],

        ITEM_TYPE: metadata_schema[ITEM_TYPE],
        'abstract': metadata_schema['abstract'],
        'headline': metadata_schema['headline'],
        'slugline': {'type': 'string'},
        'anpa_take_key': metadata_schema['anpa_take_key'],
        'keywords': metadata_schema['keywords'],
        'word_count': metadata_schema['word_count'],
        'paragraph_count': metadata_schema['word_count'],
        'priority': metadata_schema['priority'],
        'urgency': metadata_schema['urgency'],

        ITEM_STATE: metadata_schema[ITEM_STATE],
        'pubstatus': metadata_schema['pubstatus'],
        'flags': metadata_schema['flags'],
        'sms_message': metadata_schema['sms_message'],
        FORMAT: metadata_schema[FORMAT],
        'auto_publish': metadata_schema['auto_publish'],
        'assignment_id': metadata_schema['assignment_id'],
        'rewrite_of': ARCHIVE_SCHEMA_FIELDS['rewrite_of'],
        'rewritten_by': ARCHIVE_SCHEMA_FIELDS['rewritten_by'],
        'original_id': ARCHIVE_SCHEMA_FIELDS['original_id'],
        SCHEDULE_SETTINGS: ARCHIVE_SCHEMA_FIELDS[SCHEDULE_SETTINGS],
        'task': {
            'type': 'dict',
            'schema': {
                'user': Resource.rel('users'),
                'desk': Resource.rel('desks', nullable=True),
                'stage': Resource.rel('stages', nullable=True)
            }
        },
        'unique_id': metadata_schema['unique_id'],
        'unique_name': metadata_schema['unique_name'],
        'ingest_id': metadata_schema['ingest_id'],
        'family_id': metadata_schema['family_id'],
        'usageterms': metadata_schema['usageterms'],
        'copyrightnotice': metadata_schema['copyrightnotice'],
        'copyrightholder': metadata_schema['copyrightholder'],
        'profile': metadata_schema['profile'],
        BYLINE: metadata_schema[BYLINE],
        'ednote': metadata_schema['ednote'],
        'dateline': metadata_schema['dateline'],
        'expiry': metadata_schema['expiry'],
        'place': metadata_schema['place'],
        'template': metadata_schema['template'],

        'original_par_count': metadata_schema['word_count'],
        'par_count': metadata_schema['word_count'],
        'time_to_first_publish': {'type': 'integer'},
        'time_to_next_update_publish': {
            'type': 'integer',
            'default': 0
        },
        'num_desk_transitions': {
            'type': 'integer',
            'default': 0
        },
        'num_featuremedia_updates': {
            'type': 'integer',
            'default': 0
        },

        # Dictionary for statistics generated via plugins (i.e. from gen_stats_signals)
        'extra': {
            'type': 'dict',
            'mapping': not_enabled
        }
    }


class ArchiveStatisticsService(BaseService):
    def get_last_run(self):
        return self.find_one(req=None, stats_type='last_run') or {}

    def set_last_run_id(self, entry_id, last_run=None):
        if last_run is None:
            last_run = self.get_last_run()

        if last_run and last_run.get(config.ID_FIELD):
            self.patch(last_run[config.ID_FIELD], {'guid': entry_id})
        else:
            self.post([{'guid': entry_id, 'stats_type': 'last_run'}])

    def get_history_items(self, last_id, gte, item_id, chunk_size=0):
        history_service = get_resource_service('archive_history')

        last_processed_id = last_id

        while True:
            req = ParsedRequest()
            req.sort = '[("_id", 1), ("version", 1)]'

            query = {'$and': []}

            if gte:
                query['$and'].append({'_created': {'$gte': date_to_str(gte)}})

            if item_id:
                query['$and'].append({'item_id': str(item_id)})

            if last_processed_id:
                query['$and'].append({'_id': {'$gt': str(last_processed_id)}})

            req.where = json.dumps(query)

            if chunk_size > 0:
                req.max_results = int(chunk_size)

            items = list(history_service.get(req=req, lookup=None))

            if len(items) < 1:
                break

            last_processed_id = items[-1][config.ID_FIELD]
            yield items



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.emails import SuperdeskMessage

import re
import unicodedata

from email import policy
from email.encoders import encode_base64
from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart

from flask import current_app
from flask_mail import sanitize_subject, sanitize_address, sanitize_addresses, force_text, \
    formatdate

string_types = str,
text_type = str
message_policy = policy.SMTP


class AnalyticsMessage(SuperdeskMessage):
    def _message(self):
        """Creates email as 'multipart/related' instead of 'multipart/mixed'"""
        ascii_attachments = current_app.extensions['mail'].ascii_attachments
        encoding = self.charset or 'utf-8'

        attachments = self.attachments or []

        if len(attachments) == 0 and not self.html:
            # No html content and zero attachments means plain text
            msg = self._mimetext(self.body)
        elif len(attachments) > 0 and not self.html:
            # No html and at least one attachment means multipart
            msg = MIMEMultipart()
            msg.attach(self._mimetext(self.body))
        else:
            # Anything else
            msg = MIMEMultipart('related')  # This fixes embedded images in the html body
            alternative = MIMEMultipart('alternative')
            alternative.attach(self._mimetext(self.body, 'plain'))
            alternative.attach(self._mimetext(self.html, 'html'))
            msg.attach(alternative)

        if self.subject:
            msg['Subject'] = sanitize_subject(force_text(self.subject), encoding)

        msg['From'] = sanitize_address(self.sender, encoding)
        msg['To'] = ', '.join(list(set(sanitize_addresses(self.recipients, encoding))))

        msg['Date'] = formatdate(self.date, localtime=True)
        # see RFC 5322 section 3.6.4.
        msg['Message-ID'] = self.msgId

        if self.cc:
            msg['Cc'] = ', '.join(list(set(sanitize_addresses(self.cc, encoding))))

        if self.reply_to:
            msg['Reply-To'] = sanitize_address(self.reply_to, encoding)

        if self.extra_headers:
            for k, v in self.extra_headers.items():
                msg[k] = v

        SPACES = re.compile(r'[\s]+', re.UNICODE)
        for attachment in attachments:
            f = MIMEBase(*attachment.content_type.split('/'))
            f.set_payload(attachment.data)
            encode_base64(f)

            filename = attachment.filename
            if filename and ascii_attachments:
                # force filename to ascii
                filename = unicodedata.normalize('NFKD', filename)
                filename = filename.encode('ascii', 'ignore').decode('ascii')
                filename = SPACES.sub(u' ', filename).strip()

            try:
                filename and filename.encode('ascii')
            except UnicodeEncodeError:
                filename = ('UTF8', '', filename)

            f.add_header('Content-Disposition',
                         attachment.disposition,
                         filename=filename)

            for key, value in attachment.headers:
                f.add_header(key, value)

            msg.attach(f)
        if message_policy:
            msg.policy = message_policy

        return msg



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import get_resource_service
from superdesk.resource import Resource

from analytics.chart_config import ChartConfig
from analytics.base_report import BaseReportService


class PlanningUsageReportResource(Resource):
    """Planning Usage Report schema
    """

    item_methods = ['GET']
    resource_methods = ['GET']
    privileges = {'GET': 'planning_usage_report'}


class PlanningUsageReportService(BaseReportService):
    repos = ['events', 'planning', 'assignments']
    date_filter_field = '_created'
    aggregations = {
        'events': {
            'filter': {'term': {'type': 'event'}},
            'aggs': {
                'users': {
                    'terms': {
                        'field': 'original_creator',
                        'size': 0
                    }
                }
            }
        },
        'planning': {
            'filter': {'term': {'type': 'planning'}},
            'aggs': {
                'users': {
                    'terms': {
                        'field': 'original_creator',
                        'size': 0
                    }
                }
            }
        },
        'assignments': {
            'filter': {'term': {'type': 'assignment'}},
            'aggs': {
                'users': {
                    'terms': {
                        'field': 'original_creator',
                        'size': 0
                    }
                }
            }
        },
        'coverages': {
            'nested': {'path': 'coverages'},
            'aggs': {
                'users': {
                    'terms': {
                        'field': 'coverages.original_creator',
                        'size': 0
                    }
                }
            }
        }
    }

    def _get_filters(self, repos, invisible_stages):
        return None

    def generate_report(self, docs, args):
        """
        Report schema:
        {
            group: {
                [user._id]: {
                    events: [user_event_count],
                    planning: [user_planning_count],
                    coverages: [user_coverage_count],
                    assignments: [user_assignment_count]
                }
            },
            subgroup: {
                events: [event_create_count],
                planning: [planning_create_count],
                coverages: [coverage_create_count],
                assignments: [assignment_create_count],
            }
        }
        """
        aggs = (getattr(docs, 'hits') or {}).get('aggregations') or {}
        users_with_planning = self._get_users_with_planning()

        report = {
            'group': {
                user_id: {
                    'events': 0,
                    'planning': 0,
                    'coverages': 0,
                    'assignments': 0
                } for user_id in users_with_planning
            },
            'subgroup': {
                'events': 0,
                'planning': 0,
                'coverages': 0,
                'assignments': 0
            }
        }

        def add_creators(item_type):
            stats = (aggs.get(item_type) or {}).get('users') or {}

            for item in stats.get('buckets') or []:
                user_id = str(item.get('key') or '')
                if not user_id:
                    continue

                create_count = item.get('doc_count')
                if create_count < 1:
                    continue

                if user_id not in report['group']:
                    report['group'][user_id] = {
                        'events': 0,
                        'planning': 0,
                        'coverages': 0,
                        'assignments': 0
                    }

                report['group'][user_id][item_type] += create_count
                report['subgroup'][item_type] += create_count

        add_creators('coverages')
        add_creators('events')
        add_creators('planning')
        add_creators('assignments')

        return report

    def _get_users_with_planning(self):
        """Returns a list of users with the Planning privilege"""

        planning_role_ids = [
            str(role.get('_id'))
            for role in get_resource_service('roles').get(
                req=None,
                lookup={'privileges.planning': 1}
            )
        ]

        active_users = get_resource_service('users').get(
            req=None,
            lookup={'is_enabled': True}
        )

        users_with_planning = []
        for user in active_users:
            user_id = str(user.get('_id'))
            privileges = user.get('privileges') or {}

            if (privileges.get('planning') or 0) != 0:
                users_with_planning.append(user_id)
            elif str(user.get('role')) in planning_role_ids:
                users_with_planning.append(user_id)

        return users_with_planning

    def generate_highcharts_config(self, docs, args):
        params = args.get('params') or {}

        chart = params.get('chart') or {}
        chart_type = chart.get('type') or 'bar'

        report = self.generate_report(docs, args)

        chart_config = ChartConfig('planning_usage', chart_type)

        chart_config.add_source('task.user', report.get('group'))
        chart_config.add_source('planning_usage', report.get('subgroup'))

        def gen_title():
            if chart.get('title'):
                return chart['title']

            return 'Planning Module Usage'

        def gen_subtitle():
            return ChartConfig.gen_subtitle_for_dates(params)

        def get_y_axis_title():
            return 'Items Created'

        chart_config.get_title = gen_title
        chart_config.get_subtitle = gen_subtitle
        chart_config.get_y_axis_title = get_y_axis_title
        chart_config.sort_order = chart.get('sort_order') or 'desc'

        translations = args.get('translations') or {}
        chart_config.translations = translations

        report['highcharts'] = [chart_config.gen_config()]

        return report

    def generate_csv(self, docs, args):
        report = self.generate_report(docs, args)
        return report

    def generate_html(self, docs, args):
        report = self.generate_report(docs, args)
        return report



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import get_resource_service, resources
from superdesk.tests import TestCase

from analytics.planning_usage_report import init_app


class PlanningUsageReportTestCase(TestCase):
    def test_get_users_with_planning(self):
        with self.app.app_context():
            # Remove the 'Planning' app and 'planning_usage_report' endpoint if they are already configured
            try:
                self.app.settings['INSTALLED_APPS'].remove('planning')
                resources.pop('planning_usage_report', None)
            except ValueError:
                pass

            # Test PlanningUsage not registering if 'Planning' module is not configured
            init_app(self.app)
            self.assertRaises(KeyError, get_resource_service, 'planning_usage_report')

            # Test PlanningUsage registering when 'Planning' module is configured
            self.app.settings['INSTALLED_APPS'].append('planning')
            init_app(self.app)
            service = get_resource_service('planning_usage_report')

        self.app.data.insert('roles', [{
            '_id': 'role1',
            'privileges': {'planning': 0}
        }, {
            '_id': 'role2',
            'privileges': {'planning': 1}
        }])

        self.app.data.insert('users', [{
            '_id': 'user1',
            'privileges': {'planning': 0},
            'is_active': True,
            'is_enabled': True
        }, {
            '_id': 'user2',
            'privileges': {'planning': 1},
            'is_active': True,
            'is_enabled': True
        }, {
            '_id': 'user3',
            'privileges': {},
            'role': 'role1',
            'is_active': True,
            'is_enabled': True
        }, {
            '_id': 'user4',
            'privileges': {},
            'role': 'role2',
            'is_active': True,
            'is_enabled': True
        }])

        users = service._get_users_with_planning()

        self.assertEqual(users, ['user2', 'user4'])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from superdesk.utc import utc_to_local, utcnow

from analytics.stats.stats_report_service import StatsReportService
from analytics.chart_config import ChartConfig
from analytics.common import REPORT_CONFIG, CHART_TYPES

from flask import current_app as app
from datetime import timedelta


class UpdateTimeReportResource(Resource):
    """Update Time Report schema"""

    item_methods = ['GET']
    resource_methods = ['GET']
    privileges = {'GET': 'update_time_report'}


class UpdateTimeReportService(StatsReportService):
    aggregations = None
    date_filter_field = 'firstpublished'

    defaultConfig = {
        REPORT_CONFIG.CHART_TYPES: {
            # Table is the only supported chart type for this report
            # The 'report_configs' endpoint will omit all others
            CHART_TYPES.TABLE: {'enabled': True}
        },
        REPORT_CONFIG.DEFAULT_PARAMS: {
            'chart': {'type': CHART_TYPES.TABLE}
        }
    }

    def get_request_aggregations(self, params, args):
        """Disable generating aggregations"""
        return None

    def generate_elastic_query(self, args):
        query = super().generate_elastic_query(args)

        query['source']['query']['filtered']['filter']['bool']['must'].extend([
            {'range': {'time_to_next_update_publish': {'gt': 0}}},
            {'exists': {'field': 'rewritten_by'}},
        ])

        query['source']['query']['filtered']['filter']['bool']['must_not'].extend([
            {'exists': {'field': 'rewrite_of'}}
        ])

        return query

    def generate_report(self, docs, args):
        for doc in docs:
            doc.pop('stats', None)
        return docs

    def generate_highcharts_config(self, docs, args):
        items = list(self.generate_report(docs, args))

        params = args.get('params') or {}
        chart_params = params.get('chart') or {}

        title = chart_params.get('title') or 'Update Time'
        subtitle = chart_params.get('subtitle') or ChartConfig.gen_subtitle_for_dates(params)

        rows = []

        def gen_date_str(date):
            return utc_to_local(
                app.config['DEFAULT_TIMEZONE'],
                date
            ).strftime('%d/%m/%Y %H:%M')

        def gen_update_string(seconds):
            times = (utcnow().replace(minute=0, hour=0, second=0)) + timedelta(seconds=seconds)
            times = times.strftime('%H:%M').split(':')

            if int(times[0]) > 0:
                return '{} hours, {} minutes'.format(times[0], times[1])

            return '{} minutes'.format(times[1])

        for item in items:
            publish_time = item.get('firstpublished')
            update_time = publish_time + timedelta(seconds=item.get('time_to_next_update_publish'))
            updated = '{} ({})'.format(
                gen_update_string(item.get('time_to_next_update_publish')),
                gen_date_str(update_time)
            )

            rows.append([
                gen_date_str(publish_time),
                item.get('slugline') or '',
                item.get('headline') or '',
                updated
            ])

        return {
            'highcharts': [{
                'id': 'update_time_report',
                'type': 'table',
                'title': title,
                'subtitle': subtitle,
                'headers': ['Published', 'Slugline', 'Headline', 'Updated In'],
                'rows': rows
            }]
        }
